######################################################
#Collect media articles about S&P 1500 firms from Factiva#
######################################################

#Import necessary packages
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
import sys
import os.path
import time
import re
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import numpy as np

#Define desired type of content
content = "Publication" #Content options are: 'Dowjones', 'All', 'Publication', 'Website', 'Blog', 'Picture', and 'Multimedia'

#Import S&P 1500 membership data
sp = np.recfromcsv(r'C:\Users\nguest\Dropbox (MIT)\Factiva\Identify SP 1500\SP1500_feeditpy.csv', delimiter=",")

i=0
while i<len(sp):
    print('----------',i+1,'----------')
    try:
        #Define stock-specific variables needed for search form
        ctpair = str(sp[i][0])
        fds = sp[i][1].decode("utf-8")
        coname = sp[i][2].decode("utf-8")
        coname1 = sp[i][4].decode("utf-8")
        coname2 = sp[i][5].decode("utf-8")
        frm = str(sp[i][6])
        frd = str(sp[i][7])
        fry = str(sp[i][8])
        tom = str(sp[i][9])
        tod = str(sp[i][10])
        toy = str(sp[i][11])
        ctkey = 'wsj_' + ctpair

        #Define search string
        searchtext = 'fds=' + fds + ' and ' + 'hd=(' + coname1 + ') and (' + coname2 + ') and wc>49 and (rst=j or rst=wsjo)'

        #Create folder to store articles
        newpath = os.path.join(r'C:\Users\nguest\Dropbox (MIT)\Factiva\Articles\WSJ', ctpair+" "+coname)
        os.makedirs(newpath, exist_ok=True)

        #Collect articles
        j=1
        datahits=1
        while j<=datahits:
            #Find j mod 100 (100 headlines shown at a time)
            mod = j % 100

            if mod==1 or mod==51:
                tries=1
                while tries<=30:
                    try:
                        #Open browser session
                        browser = webdriver.Firefox()
                        browser.get('http://libraries.mit.edu/get/factiva')

                        #Navigate to search form
                        WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '#navmbm0 > a:nth-child(1)'))).click()
                
                    except TimeoutException:
                        tries+=1
                        browser.quit()
                        if tries==31:
                            server = smtplib.SMTP('smtp.gmail.com', 587)
                            server.starttls()
                            server.login("nguest.goog@gmail.com", "pythonemail")
                            msg = MIMEMultipart()
                            msg['Subject'] = "ERROR: No factiva access at " +coname+ " article " +str(j)
                            server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
                            server.quit()                
                        time.sleep(60)
                        continue
                    break

                #Submit search conditions

                #Ensure query genius is enabled
                qg = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@id='switchbutton']/div"))).get_attribute('class')
                if qg == 'ui-switchbutton-handle disabled':
                    WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@id='switchbutton']"))).click()
                #Enter range of dates firm was in S&P 1500
                Select(WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.ID, 'dr')))).select_by_value('Custom')
                WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '#frm'))).send_keys(frm)
                WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '#frd'))).send_keys(frd)
                WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '#fry'))).send_keys(fry)
                WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '#tom'))).send_keys(tom)
                WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '#tod'))).send_keys(tod)
                WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '#toy'))).send_keys(toy)
                #Do not remove duplicates
                Select(WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.ID, 'isrd')))).select_by_value('None')
                #Show oldest results first
                WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '#moreOptsBtn > div:nth-child(1)'))).click()
                Select(WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.ID, 'hso')))).select_by_value('PublicationDateOldestFirst')
                #Enter search string
                searchfield = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@id='assistedSearchEditorControl']/div[@id='editor']/textarea[@class='ace_text-input']")))
                searchfield.send_keys(searchtext)
                searchfield.submit()

                #Select desired content
                tab = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//span[@data-channel='" + content + "']"))).get_attribute('class')
                if tab == 'tabOff':
                    WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//span[@data-channel='" + content + "']"))).click()

                #Identify number of search results
                results = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//span[@data-channel='All']/a/span[@class='hitsCount']"))).text
                if results=='(0)':
                    datahits = 0
                    print('Search returned 0 articles for',coname)

                else:
                    datahits = int(WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.CSS_SELECTOR, '.resultsBar'))).get_attribute('data-hits'))
                    if j == 1:
                        print('Search returned',str(datahits),'articles for',coname)
                    
                    #Navigate to correct 100 articles
                    hit=mod
                    while hit<j:
                        WebDriverWait(browser, 60).until(EC.element_to_be_clickable((By.CSS_SELECTOR, '.nextItem'))).click()
                        hit+=100
                        WebDriverWait(browser, 30).until(EC.text_to_be_present_in_element((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td[@class='count']"), str(hit) + '.  '))

                    att=1
                    while att<=3:
                        try:
                            #Open article
                            WebDriverWait(browser, 60).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/a"))).click()

                            #Save article data to file
                            headline = 'Headline: ' + WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/a"))).text
                            info = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/div[@class='leadFields']"))).text
                            info1 = re.sub(' ,', ',', info)
                            newinfo = info1.split(", ")
                            if ":" in newinfo[1]: 
                                del newinfo[1]
                            if newinfo[3] == '(English)':
                                author = 'Author(s): '
                            else:
                                atext = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/div[@class='author'][last()]"))).text
                                author = 'Author(s): ' + atext
                            date = 'Date: ' + newinfo[1]
                            source = 'Source: ' + newinfo[0]
                            if newinfo[0]=='The Wall Street Journal':
                                medium = 'Medium: Print'
                                position = 'Position: ' + WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/div[last()-2]"))).text
                            else:
                                medium = 'Medium: Online'
                                position = 'Position: '
                            word_count = 'Word Count: ' + re.sub("\D", "", newinfo[2])  
                            WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/p[@class='articleParagraph enarticleParagraph']")))
                            text_elems = browser.find_elements_by_xpath("//div[@class='article enArticle']/p[@class='articleParagraph enarticleParagraph']")
                            text=""
                            for k in text_elems:
                                par = k.text
                                text = text + "\n" + par
                            article = headline + "\n" + author + "\n" + date + "\n" + source + "\n" + medium + "\n" + position + "\n" + word_count + "\n" + text
                            g=open(os.path.join(newpath, ctkey + '_' + str(j) + '.txt'), 'w', encoding='utf-8')
                            g.write(article)
                            g.close()

                        except TimeoutException:
                            print("Timeout exception", att, "at", coname, "article", j)
                            att+=1
                            if att==4:
                                server = smtplib.SMTP('smtp.gmail.com', 587)
                                server.starttls()
                                server.login("nguest.goog@gmail.com", "pythonemail")
                                msg = MIMEMultipart()
                                msg['Subject'] = "EXCEPTION: Too many timeouts at " +coname+ " article " +str(j)
                                print("EXCEPTION: Too many timeouts at", coname, "article", j)
                                server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
                                server.quit()
                            continue
                        break

                    #Move to next iteration
                    j+=1

            elif mod!=50 and mod!=0:
                att=1
                while att<=3:
                    try:
                        #Open article
                        WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/a"))).click()

                        #Save article data to file
                        headline = 'Headline: ' + WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/a"))).text
                        info = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/div[@class='leadFields']"))).text
                        info1 = re.sub(' ,', ',', info)
                        newinfo = info1.split(", ")
                        if ":" in newinfo[1]: 
                            del newinfo[1]
                        if newinfo[3] == '(English)':
                            author = 'Author(s): '
                        else:
                                atext = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/div[@class='author'][last()]"))).text
                                author = 'Author(s): ' + atext
                        date = 'Date: ' + newinfo[1]
                        source = 'Source: ' + newinfo[0]
                        if newinfo[0]=='The Wall Street Journal':
                            medium = 'Medium: Print'
                            position = 'Position: ' + WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/div[last()-2]"))).text
                        else:
                            medium = 'Medium: Online'
                            position = 'Position: '
                        word_count = 'Word Count: ' + re.sub("\D", "", newinfo[2])  
                        WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/p[@class='articleParagraph enarticleParagraph']")))
                        text_elems = browser.find_elements_by_xpath("//div[@class='article enArticle']/p[@class='articleParagraph enarticleParagraph']")
                        text=""
                        for k in text_elems:
                            par = k.text
                            text = text + "\n" + par
                        article = headline + "\n" + author + "\n" + date + "\n" + source + "\n" + medium + "\n" + position + "\n" + word_count + "\n" + text
                        g=open(os.path.join(newpath, ctkey + '_' + str(j) + '.txt'), 'w', encoding='utf-8')
                        g.write(article)
                        g.close()

                    except TimeoutException:
                        print("Timeout exception", att, "at", coname, "article", j)
                        att+=1
                        if att==4:
                            server = smtplib.SMTP('smtp.gmail.com', 587)
                            server.starttls()
                            server.login("nguest.goog@gmail.com", "pythonemail")
                            msg = MIMEMultipart()
                            msg['Subject'] = "EXCEPTION: Too many timeouts at " +coname+ " article " +str(j)
                            print("EXCEPTION: Too many timeouts at", coname, "article", j)
                            server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
                            server.quit()
                        continue
                    break

                #Move to next iteration
                j+=1

            elif mod==50:
                att=1
                while att<=3:
                    try:
                        #Open article
                        WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/a"))).click()

                        #Save article data to file
                        headline = 'Headline: ' + WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/a"))).text
                        info = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/div[@class='leadFields']"))).text
                        info1 = re.sub(' ,', ',', info)
                        newinfo = info1.split(", ")
                        if ":" in newinfo[1]: 
                            del newinfo[1]
                        if newinfo[3] == '(English)':
                            author = 'Author(s): '
                        else:
                            atext = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/div[@class='author'][last()]"))).text
                            author = 'Author(s): ' + atext
                        date = 'Date: ' + newinfo[1]
                        source = 'Source: ' + newinfo[0]
                        if newinfo[0]=='The Wall Street Journal':
                            medium = 'Medium: Print'
                            position = 'Position: ' + WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/div[last()-2]"))).text
                        else:
                            medium = 'Medium: Online'
                            position = 'Position: '
                        word_count = 'Word Count: ' + re.sub("\D", "", newinfo[2])  
                        WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/p[@class='articleParagraph enarticleParagraph']")))
                        text_elems = browser.find_elements_by_xpath("//div[@class='article enArticle']/p[@class='articleParagraph enarticleParagraph']")
                        text=""
                        for k in text_elems:
                            par = k.text
                            text = text + "\n" + par
                        article = headline + "\n" + author + "\n" + date + "\n" + source + "\n" + medium + "\n" + position + "\n" + word_count + "\n" + text
                        g=open(os.path.join(newpath, ctkey + '_' + str(j) + '.txt'), 'w', encoding='utf-8')
                        g.write(article)
                        g.close()

                    except TimeoutException:
                        print("Timeout exception", att, "at", coname, "article", j)
                        att+=1
                        if att==4:
                            server = smtplib.SMTP('smtp.gmail.com', 587)
                            server.starttls()
                            server.login("nguest.goog@gmail.com", "pythonemail")
                            msg = MIMEMultipart()
                            msg['Subject'] = "EXCEPTION: Too many timeouts at " +coname+ " article " +str(j)
                            print("EXCEPTION: Too many timeouts at", coname, "article", j)
                            server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
                            server.quit()
                        continue
                    break

                #Close browser
                browser.quit()

                #Move to next iteration
                print(j,'of',str(datahits))
                j+=1
                time.sleep(20)

            elif mod==0:
                mod = 100
                att=1
                while att<=3:
                    try:
                        #Open article
                        WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/a"))).click()

                        #Save article data to file
                        headline = 'Headline: ' + WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/a"))).text
                        info = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='headlines']/table/tbody/tr[" + str(mod) + "]/td/div[@class='leadFields']"))).text
                        info1 = re.sub(' ,', ',', info)
                        newinfo = info1.split(", ")
                        if ":" in newinfo[1]: 
                            del newinfo[1]
                        if newinfo[3] == '(English)':
                            author = 'Author(s): '
                        else:
                            atext = WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/div[@class='author'][last()]"))).text
                            author = 'Author(s): ' + atext
                        date = 'Date: ' + newinfo[1]
                        source = 'Source: ' + newinfo[0]
                        if newinfo[0]=='The Wall Street Journal':
                            medium = 'Medium: Print'
                            position = 'Position: ' + WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/div[last()-2]"))).text
                        else:
                            medium = 'Medium: Online'
                            position = 'Position: '
                        word_count = 'Word Count: ' + re.sub("\D", "", newinfo[2])  
                        WebDriverWait(browser, 30).until(EC.presence_of_element_located((By.XPATH, "//div[@class='article enArticle']/p[@class='articleParagraph enarticleParagraph']")))
                        text_elems = browser.find_elements_by_xpath("//div[@class='article enArticle']/p[@class='articleParagraph enarticleParagraph']")
                        text=""
                        for k in text_elems:
                            par = k.text
                            text = text + "\n" + par
                        article = headline + "\n" + author + "\n" + date + "\n" + source + "\n" + medium + "\n" + position + "\n" + word_count + "\n" + text
                        g=open(os.path.join(newpath, ctkey + '_' + str(j) + '.txt'), 'w', encoding='utf-8')
                        g.write(article)
                        g.close()

                    except TimeoutException:
                        print("Timeout exception", att, "at", coname, "article", j)
                        att+=1
                        if att==4:
                            server = smtplib.SMTP('smtp.gmail.com', 587)
                            server.starttls()
                            server.login("nguest.goog@gmail.com", "pythonemail")
                            msg = MIMEMultipart()
                            msg['Subject'] = "EXCEPTION: Too many timeouts at " +coname+ " article " +str(j)
                            print("EXCEPTION: Too many timeouts at", coname, "article", j)
                            server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
                            server.quit()
                        continue
                    break

                #Close browser
                browser.quit()

                #Move to next iteration
                print(j,'of',str(datahits))
                j+=1
                time.sleep(20)

        #Close browser
        browser.quit()
        
        #Send email that articles for company have been collected
        server = smtplib.SMTP('smtp.gmail.com', 587)
        server.starttls()
        server.login("nguest.goog@gmail.com", "pythonemail")
        msg = MIMEMultipart()
        msg['Subject'] = "SUCCESS: Collected " + str(datahits) + " articles for " + coname
        print("Collected", datahits, "articles for", coname)
        server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
        server.quit()
        time.sleep(20)

    #Send email if program stops because of unexpected error
    except:
        server = smtplib.SMTP('smtp.gmail.com', 587)
        server.starttls()
        server.login("nguest.goog@gmail.com", "pythonemail")
        msg = MIMEMultipart()
        msg['Subject'] = "UNEXPECTED ERROR (at " +coname+ " article " +str(j)+ ")"
        body = str(sys.exc_info()[0:1])
        msg.attach(MIMEText(body, 'plain'))
        print("UNEXPECTED ERROR (at", coname, "article", j, "):", body)
        server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
        server.quit()
        browser.quit()

    #Move to next company
    i+=1

#Send email that program has finished
server = smtplib.SMTP('smtp.gmail.com', 587)
server.starttls()
server.login("nguest.goog@gmail.com", "password")
msg = MIMEMultipart()
msg['Subject'] = "SUCCESS: Collected all articles"
print("---------- Collected all articles ----------")
server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
server.quit()





