# Read media article identifiers into dataset and perform textual analysis

#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#When opening .txt file in excel make sure to choose 'utf-8' encoding!!!!!!!!!!!!!!
#!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

import os, os.path
import sys
import re
import numpy as np
import nltk, string
from textstat.textstat import textstat
from nltk.tokenize import RegexpTokenizer
from nltk.tokenize import sent_tokenize, word_tokenize
from nltk import pos_tag
from nltk.chunk import ne_chunk
from nltk.tree import Tree
from nltk.stem.porter import PorterStemmer
from nltk.corpus import stopwords
from nltk.util import ngrams
from collections import Counter

#Load list of McDonald's "generic" and "dates and numbers" stop words expanded with 100 most common English words
stopwords = open("StopWords.txt")
stopwordsu = list(set(stopwords.read().split()))
#Load Loughran-McDonald negative words
negwords = open("LM_negwords.txt").read().split()
#http://www3.nd.edu/~mcdonald/Word_Lists.html

##Create file for storing article data
#First time
g=open('PressCoverage2020safe.txt', 'w', encoding='utf-8')
dlm='\t'
varnames = ['company_name','firm_article','date','fname','total_words','total_words_reduced','unique_words','negativity','similarity','total_q','public_q','insider_q','analyst_q','ooutsider_q','length','readability','specificity','hsmix','context']
for elem in varnames:
    g.write(elem)
    if elem not in varnames[-1]:
        g.write(dlm)
g.write('\n')

###Thereafter
##g=open('PressCoverage2020safe.txt', 'a', encoding='utf-8')
##dlm='\t'


i=1 #Company counter - start at 1
while i<=2159: #2159 companies  
    #Access folder where company articles are stored
    ispace= str(i)+" "
    folder = [folder for folder in os.listdir(r'C:\Users\nmg75\Documents\WSJ') if folder.startswith(ispace)][0]
    folderpath = os.path.join(r'C:\Users\nmg75\Documents\WSJ', folder)
    for dirpath, dirnames, files in os.walk(folderpath):
        if files:
            coname0 = re.findall(r'\\WSJ\\[0-9]+ (.+)', folderpath)[0]
            coname = coname0.replace('\\', '/')

            # Count number of files
            nfiles=0
            for file in os.listdir(folderpath):
                nfiles+=1
            
            #Analyze articles
            summ=[]
            j=1 #Company-article counter
            while j<=nfiles:
                try:
                    #Open file
                    file = open(os.path.join(folderpath, 'wsj_' + str(i) + '_' + str(j) + '.txt'), 'r', encoding='utf-8')
                    filepath=os.path.join(folderpath, 'wsj_' + str(i) + '_' + str(j) + '.txt')
                    filecontent = file.read().split("\n")
                    
                    #Save fname in dataset, prefix for all is r'C:\Users\nguest\Dropbox (MIT)\Research\007 - Reporters as Analysts\Data\Media Coverage\Articles\WSJ'
                    fname = re.findall(r'\\WSJ(.+)', filepath)[0]

                    #Date
                    date = filecontent[2].split(": ")[1]

                    #Text
                    paragraphs = filecontent[8:]
                    txt = ' '.join(str(d) for d in paragraphs)
                    #Convert all characters to lowercase
                    txtlower = txt.lower()
                    #Extract only word characters
                    twordlist = RegexpTokenizer(r'[a-z]+').tokenize(txtlower)

                    #Identify percentage of negative words
                    twords=len(twordlist)
                    nwords = len([t for t in twordlist if t in negwords])
                    if twords>0:
                        neg = round(nwords/twords, 5)
                    else:
                        neg = 0
                    
                    #Stem using the Porter algorithm
                    porter = PorterStemmer()
                    porter_stemmed = [porter.stem(token) for token in twordlist]
                    #Remove common words
                    twordlistR = [t for t in porter_stemmed if t not in stopwordsu]
                    
                    #Identify total words and unique words
                    twordsR = len(twordlistR)
                    uwordlist = list(set(twordlistR))
                    uwords = len(uwordlist)

                    #Calculate similarity to prior 5 articles
                    v=j
                    prior = -1
                    sim=[]
                    while prior >= -5:
                        if v<=5:
                            meansim=0
                        else:
                            uwordlist_w=summ[prior][0]
                            intersection= len([t for t in uwordlist if t in uwordlist_w])
                            union= len(set(uwordlist + uwordlist_w))
                            if union>0:
                                sim_v_w=intersection/union
                            else:
                                sim_v_w=0
                            sim.append(sim_v_w)
                            meansim=round(sum(sim)/len(sim), 5)
                        prior-=1

                    #Identify sentences with quotes
                    saidwords=["said","say"]
                    sentencelist=[sentence for sentence in sent_tokenize(txt)]
                    said=[]
                    for sentence in sentencelist:
                        k=[w for w in saidwords if w in sentence]
                        if len(k) > 0:
                            said.append(sentence)
                    sWquotes=[i for i in said if "\"" in i]
                    nq_all=len(sWquotes)
                    #Remove quotes from public sources
                    publicwords=["release", "statement", "conference call", "report", "announcement", "speech", "10-K", "10-Q", "8-K", "Form D", "S-1", "Form 144", "20-F", "6-K", "11-K", "DEF 14-A", "SEC filing"]
                    sWqWprivate=[]
                    nq_public=0
                    for q in sWquotes:
                        k=[w for w in publicwords if w in q]
                        if len(k) == 0:
                            sWqWprivate.append(q)
                        if len(k) > 0:
                            nq_public+=1
                    #Identify quotes from insiders, analyst, and other outsiders
                    insiderwords=["executive", "officer", "director", "president", "chairman", "CEO", "CFO", "COO"]
                    analystwords=["analyst", "broker", "brokerage"]
                    ooutsiderwords=["trader","fund manager","money manager","portfolio manager","equity strategist","equity manager","money strategist","portfolio strategist","stock strategist","manager of hedge fund","head of trading","investment strategist","investment manager","market strategist","option strategist","trading strategist","fixed-income strategist","derivatives specialist","director of portfolio strategy","money management","capital management","asset management","wealth management","professor","faculty","university","research institute","expert","economist","scientist","physician","commissioner","former","specialist"]
                    nq_insider=0
                    nq_analyst=0
                    nq_ooutsider=0
                    sWqWpWinsider=[]
                    sWqWpWanalyst=[]
                    sWqWpWooutsider=[]
                    for q in sWqWprivate:
                        insiderword=[w for w in insiderwords if w in q]
                        analystword=[w for w in analystwords if w in q]
                        ooutsiderword=[w for w in ooutsiderwords if w in q]
                        if len(ooutsiderword) > 0:
                            nq_ooutsider+=1
                            sWqWpWooutsider.append(q)
                        if len(insiderword) >= 0 and len(analystword) > 0 and len(ooutsiderword) == 0:
                            nq_analyst+=1
                            sWqWpWanalyst.append(q)
                        if len(insiderword) > 0 and len(analystword) == 0 and len(ooutsiderword) == 0:
                            nq_insider+=1
                            sWqWpWinsider.append(q)
                    #if nq_insider > 0:
                        #print('insider ' + str(nq_insider))
                        #print(sWqWpWinsider)
                    #if nq_analyst > 0:
                        #print('analyst ' + str(nq_analyst))
                        #print(sWqWpWanalyst)
                    #if nq_ooutsider > 0:
                        #print('ooutsider ' + str(nq_ooutsider))
                        #print(sWqWpWooutsider)

                    #Identify discussion of industry/economy context
                    contextwords=["industry","peer","compet","market","econom"]
                    badcontextwords=["marketing","premarket"]
                    wordlist=[word for word in word_tokenize(txt)]
                    wordlist_nobad=[w for w in wordlist if w not in badcontextwords]
                    contextlist=[]
                    for word in wordlist_nobad:
                        k=[w for w in contextwords if w in word]
                        if len(k) > 0:
                            contextlist.append(word)
                    if len(wordlist)>0:
                        context = round((len(contextlist)*100)/len(wordlist),2)
                    else:
                        context = 0

                    #Length
                    length = len(twordlist)
                    
                    #Readability
                    if textstat.lexicon_count(txt) > 0:
                        readability = round(textstat.gunning_fog(txt),1)
                    else:
                        readability = 0

                    #Specificity
                    chunked=nltk.ne_chunk(pos_tag(word_tokenize(txt)))
                    def get_continuous_chunks(txt):
                        chunked = nltk.ne_chunk(pos_tag(word_tokenize(txt)))
                        continuous_chunk = []
                        current_chunk = []
                        for i in chunked:
                            if type(i) == Tree:
                                current_chunk.append(" ".join([token for token, pos in i.leaves()]))
                            elif current_chunk:
                                named_entity = " ".join(current_chunk)
                                if named_entity not in continuous_chunk:
                                    continuous_chunk.append(named_entity)
                                    current_chunk = []
                            else:
                                continue
                        return continuous_chunk

                    named_entities=get_continuous_chunks(txt)
                    if len(named_entities) > 0:
                        specificity = round((len(named_entities)*100)/len(chunked),2)
                    else:
                        specificity = 0

                    #Hard-Soft Mix
                    no_ne_chunk = []
                    for chunk in chunked:
                        if type(chunk) != Tree:
                            no_ne_chunk.append(chunk)
                    numbers = []
                    for chunk in no_ne_chunk:
                        if chunk[1] == 'CD':
                            numbers.append(chunk[0])
                    if len(numbers) > 0:
                        hsmix = round((len(numbers)*100)/len(chunked),2)
                    else:
                        hsmix = 0
                        
                    j+=1

                    #Create unique word list summary dataset
                    summ.append([uwordlist])
                    
                    #Create row for each article
                    g.write(coname + dlm + str(j-1) + dlm + date + dlm + fname
                            + dlm + str(twords) + dlm + str(twordsR) + dlm + str(uwords) + dlm + str(neg) + dlm + str(meansim)
                            + dlm + str(nq_all) + dlm + str(nq_public) + dlm + str(nq_insider) + dlm + str(nq_analyst) + dlm + str(nq_ooutsider)
                            + dlm + str(length) + dlm + str(readability) + dlm + str(specificity) + dlm + str(hsmix) + dlm + str(context) + dlm + '\n')

                except FileNotFoundError:
                    j+=1
                
        else:
            coname0 = re.findall(r'\\WSJ\\[0-9]+ (.+)', folderpath)[0]
            coname = coname0.replace('\\', '/')
            j=1
    
    #Move to next company
    print(str(i) + ' -- ' + coname + ' (' + str(j-1) + ')')
    i+=1

#Close data file
g.close()



