######################################################
#Collect earnings press releases of S&P 1500 firms from EDGAR#
######################################################

#Import necessary packages
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.common.exceptions import TimeoutException
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
import sys
import os.path
import time
import re
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import numpy as np
import csv
import urllib.request
from html.parser import HTMLParser
import random
import traceback

#Set up html stripper
class MLStripper(HTMLParser):
    def __init__(self):
        super().__init__()
        self.reset()
        self.fed = []
    def handle_starttag(self, d, attrs):
        if d == 'table':
            self.fed.append('<'+d+'>QWERTYUIOP\n')
        #Use these tags to identify paragraphs, following WRDS
        if d in ['p','br','div','ul','li','tr']:
            self.fed.append('<par>')
        if d in ['s','c']:
            self.fed.append('<'+d+'>')
    def handle_endtag(self, d):
        if d == 'table':
            self.fed.append('<'+d+'>')
        if d in ['s','c']:
            self.fed.append('</'+d+'>')
    def handle_data(self, d):
        self.fed.append(d)
    def get_data(self):
        return ''.join(self.fed)

def strip_tags(html):
    s = MLStripper()
    s.feed(html)
    return s.get_data()

#Import List of desired EDGAR filings
fi = np.recfromcsv(r'C:\Users\nguest\Dropbox (MIT)\Research\007 - Reporters as Analysts\Data\Press Releases\SEC_EDGAR\pr_edgar.csv', delimiter=",")
prefix='http://www.sec.gov/Archives/'

#Create folder to store summary files
spath = os.path.join(r'C:\Users\nguest\Dropbox (MIT)\Research\007 - Reporters as Analysts\Data\Press Releases\SEC_EDGAR')
#os.makedirs(spath, exist_ok=True)

#Create folder to store press release text
path = os.path.join(r'C:\Users\nguest\Dropbox (MIT)\Research\007 - Reporters as Analysts\Data\Press Releases\SEC_EDGAR\CleanedFilings')
#os.makedirs(path, exist_ok=True)

#Count how many filings are press releases
prcount=0

i=0
while i<len(fi):
    try:
        #Define stock-specific variables needed for search form
        gvkey = fi[i][3].decode("utf-8")[1:]
        fname = fi[i][0].decode("utf-8")
        date = str(fi[i][1])
        exname = fi[i][2].decode("utf-8")
        fxname = gvkey + "_" + fname[11:].replace("/","_").replace("-","").replace(".txt","") + "_" + exname.replace(".","_") + ".txt"
        error_iter=0
        success=0

        while error_iter<10:
            try:
                # Grab exhibit 99 from edgar filing
                allhtml=urllib.request.urlopen(prefix+fname).read().decode('utf-8')			
                error_iter=10
                success=1
                        
            except:
                error_iter+=1

        if success==0: # Keep track of filings not available on EDGAR
            f=open(os.path.join(spath, 'file_read_errors.txt'), 'a', encoding='utf-8')
            f.write(fxname+'\n')
            f.close()

        else: # Process text
            #Extract Exhibit 99
            html = re.findall('<FILENAME>'+exname+'\s(.*?)<\/DOCUMENT>', allhtml, re.DOTALL)[0]
            html_nodesc = re.sub('<DESCRIPTION>(.+?)\n','',html)
            if len(re.findall('<DESCRIPTION>(.+?)\n', html)) > 0:
                desc = re.findall('<DESCRIPTION>(.+?)\n', html)[0]
            else:
                desc = ''

            #Clean .htm and .txt files seperately
            if len(re.findall('\.htm', exname)) > 0:
                #Parse html
                htmlp = strip_tags(html_nodesc)
                htmlp = htmlp.replace(u'\xa0', u' ')
                #Remove blank lines
                text = re.sub('\n\s*\n', '', htmlp)
                
            else: #len(re.findall('\.txt', exname)) > 0
                #Parse html (even some .txt use html)
                htmlp = strip_tags(html_nodesc)
                htmlp = htmlp.replace(u'\xa0', u' ')
                #Blank lines identify paragraphs in .txt
                text = re.sub('\n\s*\n', '<par>', htmlp)

            #Identify whether exhibit is press release
            if len(re.findall('((?:press|immediate|news|media) release|contact[^e]+?)', desc, re.I)) > 0 or len(re.findall('((?:press|immediate|news|media) release|contact[^e]+?)', text, re.I)) > 0:
                pr=1
                prcount+=1
            else:
                pr=0

            #Record which files are press releases
            record = fname +","+ date +","+ exname +","+ gvkey +","+ fi[i][4].decode("utf-8")[1:] +","+ str(pr)
            g=open(os.path.join(spath, 'pr_edgar_prdum.txt'), 'a', encoding='utf-8')
            g.write(record+'\n')
            g.close()
                
            #Remove tables (unless more than 80% alphabetic)
            notable = text.split("<table>")
            notable_div0 = [group for group in notable if len(re.sub('(QWERTYUIOP|\s)', '', group)) > 0]
            notable2 = [group for group in notable_div0 if len(re.findall('^QWERTYUIOP', group)) == 0 or len(re.sub('(QWERTYUIOP|[^a-zA-Z])', '', group))/len(re.sub('(QWERTYUIOP|\s)', '', group)) >= 0.8]
            notable3 = ''.join(notable2)
            #Remove table identifier
            notable4 = re.sub('QWERTYUIOP', '', notable3)

            #Keep only desired punctuation
            clean = re.sub('[^a-zA-Z0-9\s:;.,<>()$%?-]', '', notable4)
            
            #More cleaning
            clean0 = clean.split("<par>")
            
            #Line operations
            clean1=[]
            for par in clean0:
                par1 = re.sub('[\r\n]+', '\n', par)
                line = par1.split("\n")
                #Remove lines with <s> and <c> tags
                line1 = [n for n in line if len(re.findall('(<s>|<c>)', n)) == 0]
                #Remove lines with fewer than 20 characters or 15 alphanumeric characters
                line2 = [n for n in line1 if len(re.sub('\s', '', n)) >= 20 and len(re.sub('[^a-zA-Z0-9]', '', n)) >= 15]
                #Merge back to paragraph level
                par2 = '\n'.join(line2)
                clean1.append(par2)

            #Remove extra white space
            clean2 = [re.sub('\s+', ' ', x) for x in clean1]
            clean3 = [re.sub('^\s+', '', x) for x in clean2]

            #Remove paragraphs with fewer than 80 characters
            clean4 = [x for x in clean3 if len(re.sub('\s', '', x)) >= 80]
            #Remove paragraphs that are less than 50% alphabetic
            clean_div0 = [x for x in clean4 if len(re.sub('\s', '', x)) > 0]
            clean5 = [x for x in clean_div0 if len(re.sub('[^a-zA-Z]', '', x))/len(re.sub('\s', '', x)) >= 0.5]

            #Insert blank line between paragraphs
            cleanfinal = '\n\n'.join(clean5)

            if pr==1:
                #Write to document
                h=open(os.path.join(path, fxname), 'w', encoding='utf-8')
                h.write(cleanfinal)
                h.close()

    #Send email if program stops because of unexpected error
    except Exception as e:
        body = traceback.format_exc()
        print("UNEXPECTED ERROR AT FILING " + str(i+1))
        print(body)
        server = smtplib.SMTP('smtp.gmail.com', 587)
        server.starttls()
        server.login("nguest.goog@gmail.com", "pythonemail")
        msg = MIMEMultipart()
        msg['Subject'] = "UNEXPECTED ERROR AT FILING " + str(i+1)
        msg.attach(MIMEText(body, 'plain'))
        server.sendmail('nguest.goog@gmail.com','nguest@mit.edu',msg.as_string())
        server.quit()

    i+=1
     
    if i % 100 == 0:
        print('Filings:' + str(i) + ', Press Releases:' + str(prcount))


