* Code for "The Information Role of the Media in Earnings News" by Nick Guest (2020)

This file contains the code that converts raw data into the final dataset used in the analyses.
It also provides the code that produces the key results in the paper.
;


%let wrds = wrds.wharton.upenn.edu 4016; 
options comamid=TCP remote=WRDS;
signon username=_prompt_;
Libname rwork slibref=work server=wrds;

libname mydir 'C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Analysis';
libname ta 'C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Analysis\TextAnalysis';
libname pr 'C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Press Releases\SEC_EDGAR';
libname instown "C:\Users\nmg75\Dropbox (Personal)\Research\Data_Miscellaneous\Institutional ownership";
libname sp1500 'C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Analysis\Identify SP 1500';
libname iclink 'C:\Users\nmg75\Dropbox (Personal)\Research\Programming-Methodology';
libname mytaq "C:\Users\nmg75\Dropbox (Personal)\Research\Data_Miscellaneous\TAQ";
libname ancerno "C:\Users\nmg75\Dropbox (Personal)\Research\Data_Miscellaneous\Ancerno\TradeData";
libname rpna "C:\Users\nmg75\Dropbox (Personal)\Research\Data_Large\Ravenpack\CleanData";







*********************************************************************************************************************************;
*********************************************************************************************************************************;
*MACROS;
*********************************************************************************************************************************;
*********************************************************************************************************************************;


*Winsorization/Truncation macro;

%macro WT(data=_last_, out=, byvar=none, vars=, type = W, pctl = 1 99, drop= N);
 
      %if &out = %then %let out = &data;
   
      %let varLow=;
      %let varHigh=;
      %let xn=1;
 
      %do %until (%scan(&vars,&xn)= );
      %let token = %scan(&vars,&xn);
      %let varLow = &varLow &token.Low;
      %let varHigh = &varHigh &token.High;
      %let xn = %EVAL(&xn + 1);
      %end;
 
      %let xn = %eval(&xn-1);
 
      data xtemp;
            set &data;
 
      %let dropvar = ;
      %if &byvar = none %then %do;
            data xtemp;
            set xtemp;
            xbyvar = 1;
 
      %let byvar = xbyvar;
      %let dropvar = xbyvar;
      %end;
 
      proc sort data = xtemp;
            by &byvar;
 
      /*compute percentage cutoff values*/
      proc univariate data = xtemp noprint;
      by &byvar;
      var &vars;
      output out = xtemp_pctl PCTLPTS = &pctl PCTLPRE = &vars PCTLNAME = Low High;
 
      data &out;
      merge xtemp xtemp_pctl; /*merge percentage cutoff values into main dataset*/
      by &byvar;
      array trimvars{&xn} &vars;
      array trimvarl{&xn} &varLow;
      array trimvarh{&xn} &varHigh;
 
      do xi = 1 to dim(trimvars);
                  /*winsorize variables*/
            %if &type = W %then %do;
                  if trimvars{xi} ne . then do;
                        if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = trimvarl{xi};
                        if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = trimvarh{xi};
                  end;
            %end;
                  /*truncate variables*/
            %else %do;
                  if trimvars{xi} ne . then do;
                        if (trimvars{xi} < trimvarl{xi}) then trimvars{xi} = .T;
                        if (trimvars{xi} > trimvarh{xi}) then trimvars{xi} = .T;
                  end;
            %end;
 
                  %if &drop = Y %then %do;
                     if trimvars{xi} = .T then delete;
                  %end;
 
            end;
      drop &varLow &varHigh &dropvar xi;
 
      /*delete temporary datasets created during macro execution*/
      proc datasets library=work nolist;
            delete xtemp xtemp_pctl; quit; run;
 
%mend;


%macro dummy( 
   data=_last_ ,    /* name of input dataset                  */
   out=&data,       /* name of output dataset                 */
   var= ,           /* variable(s) to be dummied              */
   base=_last_,     /* base category                          */
   prefix = D_,     /* prefix for dummy variable names        */
   format =,        /* format used to categorize variable     */
   name  = VAL,     /* VAL: variable names are D_value        */
   fullrank=1       /* Eliminate dummy for baseline category? */
   );

	%let abort = 0;
   %if (%length(&var) = 0) %then %do;
       %put ERROR: DUMMY: VAR= must be specified;
		 %let abort=1;
       %goto done;
       %end;

%let base = %upcase(&base);
%let name = %upcase(&name);

%if %upcase(&data) = _LAST_ %then %let data = &syslast;
%if %upcase(&data) = _NULL_ %then %do;
	%put ERROR: There is no default input data set (_LAST_ is _NULL_);
	%let abort=1;
	%goto DONE;
	%end;
	
options nonotes;

%*-- Initialize output data set;
%if &out ^= &data %then %do;
	data &out;
		set &data;
	%end;
	
%let prefix = %upcase(&prefix);

%*-- j indexes variables, vari is the current variable name;
%local j vari;
%let j=1;
%*-- Find the current variable name;
%let vari= %scan(&var,    &j, %str( ));

%******************************************************************;
%*-- Loop over variables; 
%******************************************************************;
%do %while(&vari ^= );

	%*-- Find the current prefix for dummies;
	%let pre = %scan(&prefix, &j, %str( ));
	%if &pre = VARNAME | &pre = %then %let pre=&vari._;
	%*-- Keyword BLANK for prefix indicates no prefix;
	%if &pre=BLANK %then %let pre=;

	%*-- Find the current base for dummies;
	%let baseval = %scan(&base, &j, %str( ));
	%if &baseval = %then %let baseval=_LAST_;

	%*-- Find the current format for dummies;
	%let fmt = %scan(&format, &j, %str( ));

*-- determine values of variable to be dummied;
proc summary data = &out nway ;
     class &vari ;
     %if %length(&fmt) gt 0 %then %do;
	  		%*-- Make sure format name includes a '.';
        %if "%substr(&fmt, %length(&fmt))" ne "." 
		  		%then %let fmt = &fmt..;
        format &vari &fmt;
     %end;
     output out = _cats_ ( keep = &vari ) ;
	%if &syserr > 4 %then %let abort=1; 
	%if &abort %then %goto DONE;

	%if &fullrank %then %do;
	*-- Eliminate the base category;
	data _cats_;
		set _cats_ end=_eof_;
		%if &baseval = _FIRST_ | &baseval = LOW 
			%then %str( if _n_ = 1 then delete;);
		%else %if &baseval = _LAST_ | &baseval = HIGH
			%then %str( if _eof_ then delete;);
		%else %str(if &vari = &baseval then delete;);
	run;
	%end;

data _null_ ;
 set _cats_ nobs = numvals ;

 if _n_ = 1 then do;
	%*-- If there are no non-baseline values - abort macro; 
	call symput('abort',trim( left( put( (numvals=0), best. ) ) ) ) ;
	%*-- Place number of dummies into macro variable num;
	call symput( 'num', trim( left( put( numvals, best. ) ) ) ) ;
	end;

	%*-- Number the values, place in macro variables c##; 
	%if %length(&fmt) gt 0 %then %do;
		call symput ( 'c' || trim ( left ( put ( _n_,     best. ) ) ),
								trim(left(put(&vari,&fmt)) ) );
	%end;
	%else %do;
	call symput ( 'c' || trim ( left ( put ( _n_,     best. ) ) ),
								trim ( left ( &vari ) ) ) ;
	%end;
run ;
%if &syserr > 4 %then %let abort=1; %if &abort %then %goto DONE;

proc datasets library=work nolist; delete _cats_; run;

%******************************************************************;
%* Create list of dummy variables for the j-th input variable;
%******************************************************************;

%if "&name" = "VAL" %then %do ;
	%*-- Names by variable value;
	%let vl&j =; 
	%do k=1 %to &num;
		%if %sysevalf(&sysver  < 7 & %length(&pre&&c&k) > 8) %then %do;
			%put ERROR: Cannot generate names longer than 8 characters;
			%let abort=1;
			%goto DONE;
			%end;
		%let vl&j = &&vl&j  &pre&&c&k;
		%end; ;
%*put vl&j = &&&vl&j;

data &out;
	set &out ;
	
	array __d ( &num ) %do k=1 %to &num ;	&pre&&c&k
							%end ; ;
	%put DUMMY: Creating dummy variables &pre&&c1 .. &pre&&c&num for &vari;
	%end ;

%else %do ;
	%*-- Numeric suffix names;
	%let vl&j =; 
	%do k=1 %to &num; 
		%if %sysevalf(&sysver  < 7 & %length(&pre.&k) > 8) %then %do;
			%put ERROR: Cannot generate names longer than 8 characters;
			%let abort=1;
			%goto endloop;
			%end;
		%let vl&j = &&vl&j  &pre.&k;
		%end; ;
%*put vl&j = &&&vl&j;
run;
	
%******************************************************************;
%* Assign values to the dummy variables for the j-th input variable;
%******************************************************************;
data &out  ( rename = ( %do k=1 %to &num ;
						d&k = &pre.&k
						%end ; ) ) ;
	set &out ;
	%put DUMMY: Creating dummy variables &pre.1 .. &pre.&num;
	array __d ( &num ) d1-d&num ;
	%end ;

	%*---------------------------------------------------------;
   %*   Handle missing values (for V7+ only);
	%*-- (to do this for V6.12 requires separate processing for
	      character and numeric variables);
	%*---------------------------------------------------------;
	%if %sysevalf(&sysver  >= 7) %then %do;
     if missing(&vari) then do;
	  	 do j=1 to &num;
        __d(j)=.;
		  end;
		return;
     end;
	%end;

	%*---------------------------------------------------------;
   %*   Assign values to dummy variables;
	%*---------------------------------------------------------;
	drop j;
	do j = 1 to &num ; /* initialize to 0 */
		__d(j) = 0 ;
	end ;


     %if %length(&fmt) eq 0 %then %do;
     %*-- Case 1:  No format;
        if &vari = "&c1" then __d ( 1 ) = 1 ;  /* create dummies */
        %do i = 2 %to &num ;       
           else if &vari="&&c&i" then __d ( &i ) = 1 ;
        %end;
     %end;

     %else %do;
     %*-- Case 2:  with format;
        if put(&vari,&fmt) = "&c1" then __d ( 1 ) = 1 ;
        %do i = 2 %to &num ;       
           else if put(&vari,&fmt)="&&c&i" then __d ( &i ) = 1;
        %end;
     %end;
run ;

%*-- Find the next variable;

%let j=%eval(&j+1);
%let vari = %scan(&var, &j, %str( ));

%*put End of loop(&i): vari = &vari  pre=&pre;
%endloop:
%end;  /* %do %while */

%done:
%if &abort %then %put ERROR: The DUMMY macro ended abnormally.;
options notes;
%mend dummy ;




%macro CEM (
			lib = work,
			dataset = input,
			ID = id ,
			treat = treat,
			keep = varlist,
			DEL_miss = 0,
			MATCH_TYPE = N,
			method = Sturges,
			path_graph = path_graph,
			report = on
			);

data L1_TOT;
winchester=1;
run;

/*begin "keep" option*/
%if "&keep" ^= "" %then %do; 
data keepdataset;
set &lib..&dataset.;
keep &keep. &treat. &id.;
run;
%end;

%if "&keep" = "" %then %do; 
data keepdataset;
set &lib..&dataset.;
run;
%end;
/*end "keep" option*/

proc freq data=keepdataset;
table &treat /OUT = COUNT_TREAT_UNT;
run;


%if "&method" ^= "Sturges" %then %do;  /*if the binning option is Percent then skip*/

%CEM (
lib = &lib,
dataset = &dataset,
ID = &ID,
treat = &treat,
keep = &keep,
DEL_miss = &DEL_miss,
MATCH_TYPE = &MATCH_TYPE,
method = Sturges,
path_graph = &path_graph,
report = off
);


data keepdataset;
	set keepdataset;
	drop Wsc1_stur matched;
run;
%end;


%if "&method" ^= "Sturges" %then %do;  /*if the binning option is Percent then skip*/
%let vvv1 = 4;
%let vvv2 = 5;
%let vvv3 = 10;
%let vvv4 = 100;
data strata;
format strata $56.;
%do HHH = 1 %to 4;
strata = STRIP("&&vvv&HHH.");
output;
%end;
run;
%end;

%if "&method" = "Sturges" %then %do;  /*if the binning option is Percent then do*/
%let vvv1 = Stur;
data strata;
format strata $56.;
%do HHH = 1 %to 1;
strata = STRIP("&&vvv&HHH.");
output;
%end;
run;
%end;


/*****************************************/
/* RECODING OF INPUT VARIABLE            */
/*****************************************/
proc datasets; contents data= keepdataset out= Name_D (keep = NAME TYPE);run;
/*****************************************/
/* CHECK ON TREAT VARIABLE               */
/*****************************************/
data _NULL_;
set Name_D;
if upcase(NAME) = upcase("&Treat.") then call symput ("type_check",type);
run;
%if &type_check. = 2 %then %do;
%put ERROR: TREATMENT VARIABLE HAS TO BE NUMERICAL;
%ABORT;
%end;
proc sort data=keepdataset (keep = &Treat.) out=check_value nodupkey; by &Treat.; run;
data check_value;
set check_value;
rename &Treat. = value;
num = _N_;
run;
proc sql;
select max(num), max(value), min(value) into :observation, :Max_1, :Min_1
from check_value;
quit;
%if &observation. ne 2 %then %do;
%put ERROR: TREATMENT VARIABLE REQUIRES ONLY TWO DIFFERENT VALUES;
%ABORT;
%end;
%if &max_1. ne 1 AND &min_1 ne 0 %then %do;
data keepdataset;
set keepdataset;
if &Treat. = &Max_1. then &Treat = 1;
else &Treat = 0;
run;
%put WARNING: TREAT VARIABLE IS RECODED WITH DEFAULT VALUES 0 (MIN VALUE) AND 1 (MAX VALUE);
%end;
proc sort data=keepdataset out=control nodupkey; by &id; run;
data _NULL_;
set keepdataset;
call symput ('n_origine', _N_);
run;
data _NULL_;
set control;
call symput ('n_nodupkey', _N_);
run;
%IF %eval(&n_origine.) = %eval(&n_nodupkey.) %THEN %DO;
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* RECODING OF INPUT VARIABLES*/
/*&&&&&&&&&&&&&&&&&&&&&&&&&&&&*/
proc datasets ;
contents data= keepdataset out=Name_D(keep=NAME TYPE);
run;
data Name_D_cat Name_D_num;
set Name_D;
if upcase(NAME) = upcase("&ID.") then delete;
if upcase(NAME) = upcase("&TREAT.") then delete;
if TYPE=1 then output Name_D_num;
if TYPE=2 then output Name_D_cat;
run;
DATA TRANSCOD_CAT;
SET Name_D_CAT;
FORMAT TRANS_CAT $4.;
TRANS_CAT = COMPRESS("C_"||_N_);
RUN;
DATA TRANSCOD_NUM;
SET Name_D_NUM;
FORMAT TRANS_NUM $4.;
TRANS_NUM = COMPRESS("V_"||_N_);
RUN;

/******************************************************************************************/
/* CHECKING IF CATEGORICAL VARIABLES ARE EXACTLY ASSOCIATED WITH TREATMENT 				  */
/******************************************************************************************/

%let opcat = %sysfunc(open(Name_D_CAT,i));
%let num_cat = %sysfunc(attrn(&opcat.,nobs));
%let clcat = %sysfunc(close(&opcat.));
%if %eval(&num_cat.)>0 %then %do;

data _NULL_;
	set name_d_cat;
	call symput ("check", _N_);
	call symput (compress("namecat"||_N_), trim(left(NAME)));
run;

%DO v = 1 %to &num_cat.;
proc freq data = keepdataset; table &&namecat&v. * &treat. /out=CheckCat; run;
proc transpose data=CheckCat out=CheckCat1 (drop = _name_ _label_) prefix=Tr;
by &&namecat&v.;
id &treat.;
var COUNT;
run;
data CheckCat2;
	set CheckCat1;
	somma = Tr0 + Tr1;
run; 
proc means data=CheckCat2 sum noprint nway;
output out=CheckCat3 (drop = _type_ _freq_) sum(somma)=somma1;
run;
data _NULL_;
	set CheckCat3;
	call symput ('som', somma1);
run;
%if &som. <=0 %then %do;
%put ERROR: YOU HAVE INSERT A CATEGORICAL VARIABLE EXACTLY ASSOCIATED WITH THE TREATMENT VARIABLE!;
%ABORT;
%end;
%end;
%end;


/******************************************************************************************/
/* DELETING MISSING VALUES WHEN THE MACRO PARAMETER DEL_MISS IS ON		 				  */
/******************************************************************************************/

%if &del_miss. = 1 %then %do;
/*CREATE A COPY OF THE ORIGINAL DATASET. 
THE DATASET SUBMITTED TO THE MACRO WILL BE REDUCED DELETING ALL MISSING RECORDS*/
data keepdataset_Original;
	set keepdataset;
run;

/*CATEGORICAL MISSING VALUES*/
%let opcat = %sysfunc(open(Name_D_CAT,i));
%let num_cat = %sysfunc(attrn(&opcat.,nobs));
%let clcat = %sysfunc(close(&opcat.));
%if %eval(&num_cat.)>0 %then %do;
data _NULL_;
	set name_d_cat;
	call symput ("check", _N_);
	call symput (compress("namecat"||_N_), trim(left(NAME)));
run;

data keepdataset; 
	set keepdataset; 
	%do v1 = 1 %to &num_cat.;
	if &&namecat&v1. = "" then delete;
	%end;
run;
%end;

/*NUMERICAL MISSING VALUES*/
%let opNUM = %sysfunc(open(Name_D_NUM,i));
%let num_NUM = %sysfunc(attrn(&opNUM.,nobs));
%let clNUM = %sysfunc(close(&opNUM.));
%if %eval(&num_NUM.)>0 %then %do;
data _NULL_;
	set name_d_NUM;
	call symput ("check", _N_);
	call symput (compress("nameNUM"||_N_), trim(left(NAME)));
run;
data keepdataset; 
	set keepdataset; 
	%do w1 = 1 %to &num_NUM.;
	if &&nameNUM&w1. = . then delete;
	%end;
run;
%end;
%end;

/********************************************************************************/
/* L1 - MACRO																	*/
/********************************************************************************/
%L1 (CODICE = 0,
ID1 = &id,
TREAT1 = &Treat,
COD_DATASET=keepdataset);
data L1_TOT;
format codice $20.;
set L1_TOT L1_0;
run;
/********************************************************************************/
/********************************************************************************/

%let opnum = %sysfunc(open(Name_D_num,i));
%let numtot = %sysfunc(attrn(&opnum.,nobs));
%let cltranscat = %sysfunc(close(&opnum.));
%if %eval(&numtot.)>0 %then %do;
/*ONLY NUM*/
data _NULL_;
set Name_D_num;
call symput ('numtot', _N_);
call symput (compress('var_'||_N_),trim(left(NAME)));
run;
%end;
%put ************************************************************************************************;
%put &numtot. number of numeric variables considered for stratification and matching 		     	 ;
%put ************************************************************************************************;

%put ************************************************************************************************;
%put method is &method										    									 ;
%put ************************************************************************************************;

%if "&method" ^= "Sturges" %then %do;  /*if the binning option is Percent then do*/

%put ************************************************************************************************;
%put the binning option is Percent instead of Sturges!						   						 ;
%put ************************************************************************************************;

%if &numtot. ne 0 %then %do;
proc rank data=keepdataset out=ranked4 (keep = %do i = 1 %to &numtot;
&&var_&i
%end;
%do i = 1 %to &numtot;
v_&i._4
%end;
&Treat
&ID)
groups=4;
var
%do i = 1 %to &numtot;
&&var_&i
%end;
;
ranks
%do i = 1 %to &numtot;
V_&i._4
%end;
;
run;
proc rank data=keepdataset
out=ranked5 (keep = %do i = 1 %to &numtot;
&&var_&i
%end;
%do i = 1 %to &numtot;
v_&i._5
%end;
&ID
&Treat)
groups=5;
var
%do i = 1 %to &numtot;
&&var_&i
%end;
;
ranks
%do i = 1 %to &numtot;
V_&i._5
%end;
;
run;
proc rank data=keepdataset
out=ranked10 (keep = %do i = 1 %to &numtot;
&&var_&i
%end;
%do i = 1 %to &numtot;
v_&i._10
%end;
&ID
&Treat)
groups=10;
var
%do i = 1 %to &numtot;
&&var_&i
%end;
;
ranks
%do i = 1 %to &numtot;
V_&i._10
%end;
;
run;
proc rank data=keepdataset
out=ranked100 (keep = %do i = 1 %to &numtot;
&&var_&i
%end;
%do i = 1 %to &numtot;
v_&i._100
%end;
&ID
&Treat)
groups=100;
var
%do i = 1 %to &numtot;
&&var_&i
%end;
;
ranks
%do i = 1 %to &numtot;
V_&i._100
%end;
;
run;
PROC SORT DATA=RANKED4; BY &ID; RUN;
PROC SORT DATA=RANKED5; BY &ID; RUN;
PROC SORT DATA=RANKED10; BY &ID; RUN;
PROC SORT DATA=RANKED100; BY &ID; RUN;
DATA rec_v_num ;
MERGE RANKED4 RANKED5 RANKED10 RANKED100;
BY &ID;
RUN;
%end;
%end;

%if "&method" = "Sturges" %then %do;

%put ************************************************************************************************;
%put the default binning option is Sturges!							     ;
%put ************************************************************************************************;

data _NULL_;
	set keepdataset;
	call symput ('n', _N_);
run;

%let Z=%sysevalf(1+%sysfunc(log2(%eval(&n))), floor); /*Z is the rounding of Sturges*/
%let Z1=%sysevalf(1+%sysfunc(log2(%eval(&n))), floor); /*Z1 is the rounding of Sturges*/

/*Calculate for each Numerical Variable StdDev Min and Max*/
%do b = 1 %to &numtot;
	proc means data=keepdataset (keep = &&var_&b) stddev min max noprint nway;
		output out=stock_&&var_&b (drop = _type_ _freq_)  min(&&var_&b)=min_&&var_&b
														  max(&&var_&b)=max_&&var_&b;
	run;
	/*Stock them*/
	data _NULL_;
		set stock_&&var_&b;
		call symput ("min_&&var_&b", trim(left(min_&&var_&b)));
		call symput ("max_&&var_&b", trim(left(max_&&var_&b)));
	run;

%let W_&&var_&b = %sysevalf( ((&&&&max_&&var_&b - &&&&min_&&var_&b)  / &Z. )); /*Width of each bin*/
/*%let W1_&&var_&b = %sysfunc( (round(&&&&W_&&var_&v, .01) ));*/

%end;

/*Create New Variables based on the binning above*/
data rec_v_num (keep = %do i = 1 %to &numtot;
&&var_&i
%end;
%do i = 1 %to &numtot;
v_&i._Stur
%end;
&ID
&Treat);
	set keepdataset;
	%do v = 1 %to &numtot;
		if &&var_&v >= %sysevalf(&&&&min_&&var_&v) AND &&var_&v <= %sysevalf((&&&&min_&&var_&v + &&&&W_&&var_&v)) then V_&v._Stur = 1;
		%do s = 2 %to &Z;
		if &&var_&v > %sysevalf( (&&&&min_&&var_&v + %sysevalf((&s-1)* &&&&W_&&var_&v))) AND &&var_&v <= %sysevalf( (&&&&min_&&var_&v + %sysevalf((&s)* &&&&W_&&var_&v))) then V_&v._Stur = &s;
		/*if &&var_&v >= %sysfunc(round( (&&&&min_&&var_&v + %sysevalf((&s-1)* &&&&W_&&var_&v)) ).1 ) then V_&v._Stur = &s;*/
	%end;
		%end;
run;

%end;


/*ONLY CAT*/
%let dsidtot1 = %sysfunc(open(Name_D_cat,i));
%let numtot1 = %sysfunc(attrn(&dsidtot1.,nobs));
%let rktot1 = %sysfunc(close(&dsidtot1.));
%if &numtot1. ne 0 %then %do;
%do r=1 %to &numtot1.;
%let dsid2 = %sysfunc(open(Name_D_cat,i));
%let varnum2 = %sysfunc(varnum(&dsid2.,NAME));
%let rc2 = %sysfunc(fetchobs(&dsid2.,&r.));
%let label2 = %sysfunc(getvarc(&dsid2.,&varnum2.));
%let rk2 = %sysfunc(close(&dsid2.));
proc freq data=keepdataset noprint;
tables &label2. /out=V_CAT_&r.(DROP=COUNT PERCENT);
run;
quit;
data V_CAT_&r.;
set V_CAT_&r.;
format c_&r $32.;
if &label2. = "" then delete;
run;
data V_CAT_&r.;
set V_CAT_&r.;
format c_&r $32.;
c_&r = left(_n_);
run;
%if &DEL_miss. =0 %then %do;
data V_CAT_&r._app;
format c_&r $32.;
&label2. ="";
c_&r ="MISS" ;
run;
data V_CAT_&r.;
format c_&r $32. &label2. $32.;
set V_CAT_&r._app V_CAT_&r.;
run;
%end;
proc sort data=V_CAT_&r.;
by &label2.;
run;
%let opnum1 = %sysfunc(open(TRANSCOD_NUM,i));
%let k = %sysfunc(attrn(&opnum1.,nobs));
%let clnum = %sysfunc(close(&opnum1.));
%if &k.>0 %then %do;
data _NULL_;
set TRANSCOD_NUM;
call symput ('k',_N_);
call symput (compress('v'||_N_),trim(left(name)));
run;
%end;
%if &r.=1 %then %do;
proc sort data=keepdataset;
by &label2.;
run;
%if &k.>0 %then %do;
data REC_V_CAT
(drop = %do k = 1 %to &k;
&&v&k
%end;
)
;
merge V_CAT_&r. (in=a) keepdataset (in=b);
by &label2.;
if a and b;
run;
%end;
%else %do;
data REC_V_CAT ;
merge V_CAT_&r. (in=a) keepdataset (in=b);
by &label2.;
if a and b;
run;
%end;
%end;
%if &r.>=2 %then %do;
proc sort data=REC_V_CAT;
by &label2.;
run;
data REC_V_CAT;
merge V_CAT_&r.(in=a) REC_V_CAT (in=b);
by &label2.;
if a and b;
run;
%end;
%end;
%end;
/*%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* updating zero cat or num */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%*/
%let dsid_100 = %sysfunc(open(rec_v_cat ,i));
%let rk_100 = %sysfunc(close(&dsid_100.));
%let dsid_101 = %sysfunc(open(rec_v_num ,i));
%let rk_101 = %sysfunc(close(&dsid_101.));
%if &dsid_100. gt 0 and &dsid_101. gt 0 %then %do;
PROC SQL;
CREATE TABLE REC_DATA AS SELECT 
* 
FROM rec_v_cat AS A
inner join rec_v_num AS B
ON A.&id. = B.&id.;
Quit;
%end;
%else %if &dsid_100. = 0 and &dsid_101. gt 0 %then %do;
PROC
SQL;
CREATE TABLE REC_DATA AS SELECT
*
FROM rec_v_num
;
Quit;
%end;
%else %if &dsid_100. gt 0 and &dsid_101. = 0 %then %do;
PROC SQL;
CREATE TABLE REC_DATA AS SELECT
*
FROM rec_v_cat
;
Quit;
%end;
%let dsidtot2 = %sysfunc(open(Name_D_num,i));
%let numtot2 = %sysfunc(attrn(&dsidtot2.,nobs));
%let rktot2 = %sysfunc(close(&dsidtot2.));
%let dsidtot3 = %sysfunc(open(Name_D_cat,i));
%let numtot3 = %sysfunc(attrn(&dsidtot3.,nobs));
%let rktot3 = %sysfunc(close(&dsidtot3.));
%put ***********************************************************************************************;
%put &numtot2. number of numeric variables considered for stratification and matching 		    ;
%put ***********************************************************************************************;
%put ***********************************************************************************************;
%put &numtot3. number of categorical variables considered for stratification and matching           ;
%put ***********************************************************************************************;
%if %eval(&numtot2.)>0 %then %do;
%do i=1 %to &numtot2. ;
data bin_&i;
format strata_&i $350.;
set strata (rename=(strata=strata_&i));
strata_&i=compress("V_&i._"||strata_&i);
run;
%end;
%do i=1 %to %eval(&numtot2.-1) ;
%let j = &numtot2.;
proc sql;
create table strata&i as select 
bin_&i..strata_&i
,bin_&j..strata_&j
from bin_&i
,bin_&j;
quit;
data bin_&j;
set strata&i;
format strata_&j $350.;
strata_&j=compress(strata_&i||"||"||strata_&j);
drop strata_&i;
run;
%end;
%end;
data _NULL_;
set TRANSCOD_CAT;
call symput ('ncat', _N_);
call symput (compress('cat'||_N_), trim(left(trans_cat)));
run;
%if %eval(&numtot3.)>0 AND %eval(&numtot2.)>0 %then %do;
data Trans_cod;
set bin_&numtot2.;
codcat = compress(
%do c = 1 %to &ncat;
"&&cat&c" || "||" ||
%end;
" ");
strata_finale = compress(codcat||strata_&numtot2.);
run;
%end;
%if %eval(&numtot3.)>0 AND %eval(&numtot2.)=0 %then %do;
data Trans_cod;
%if %eval(&numtot3.)=1 %then %do;
strata_finale = compress(
%do c = 1 %to &ncat;
"&&cat&c"
%end;
);
%end;
%if %eval(&numtot3.)>1 %then %do;
strata_finale = compress(
%do c = 1 %to &ncat;
"&&cat&c"||"||"||
%end;
"");
%end;
run;
%end;
%if %eval(&numtot3.)=0 AND %eval(&numtot2.)>0 %then %do;
data Trans_cod;
set bin_&numtot2.;
strata_finale = compress(strata_&numtot2.);
run;
%end;
DATA _null;
SET Trans_cod;
CALL SYMPUT ('NCOD', _N_);
call symput (compress('Cod_'||_N_), trim(left(strata_finale)));
run;
%if %eval(&ncod)>100 %then
%do; %let maxcod = 10; %end; /*!!!RIMETTERE A 99*/
%else
%do; %let maxcod = &ncod; %end;
/**********************************************/
/* MATCHING ONE-TO-ONE 			      */
/**********************************************/
%if &MATCH_TYPE. = ONE %then %do;

%put ***********************************************************************************************;
%put The number of automatic rounds is ---> &maxcod.			         		    ;
%put ***********************************************************************************************;

%do i=1 %to &maxcod.;
%if %eval(&numtot3.)>1 AND %eval(&numtot2.)=0 %then %do;
data cod_strata;
set rec_data;
cod_num_&i. = compress(&&cod_&i "");
run;
%end;
%if %eval(&numtot3.)=1 AND %eval(&numtot2.)=0 %then %do;
data cod_strata;
set rec_data;
cod_num_&i. = compress(&&cod_&i);
run;
%end;
%if %eval(&numtot2.)>0 %then %do;
data cod_strata;
set rec_data;
cod_num_&i. = compress(&&cod_&i||"");
run;
%end;
PROC FREQ DATA=cod_strata NOPRINT;
TABLES COD_NUM_&i.*&treat / OUT=FREQ_&i.(DROP=PERCENT);
run;
proc transpose data=FREQ_&i. out=FREQ_&i.(drop=_NAME_ _LABEL_) prefix=treat;
by COD_NUM_&i.;
id &treat.;
var COUNT;
run;
data FREQ_&i.;
set FREQ_&i.;
if treat0 = . then treat0=0;
if treat1 = . then treat1=0;
_NSIZE_ = min(treat0, treat1);
run;
data untreated(DROP=&treat) treated(DROP=&treat);
set cod_strata (keep=&ID. COD_NUM_&i. &treat);
if &treat = 0 then output untreated;
if &treat = 1 then output treated;
run;
proc sort data=untreated; by COD_NUM_&i.; run;
proc surveyselect data=untreated out=sample_untreated_&i. (drop= SelectionProb SamplingWeight) method=srs sampsize=FREQ_&i.;
strata COD_NUM_&i.;
id &id.;
run;
proc sort data=treated; by COD_NUM_&i.; run;
proc surveyselect data=treated out=sample_treated_&i. (drop= SelectionProb SamplingWeight) method=srs sampsize=FREQ_&i.;
strata COD_NUM_&i.;
id &id.;
run;

data dataset_matched_&i.;
set sample_treated_&i. sample_untreated_&i.;
matched=1;
run;


/*************** CHECK # OF MATCHED UNITS ********************/
%let dtstmatch = %sysfunc(open(dataset_matched_&i.,i));
%let nmatched = %sysfunc(attrn(&dtstmatch.,nobs));
%let dtsm = %sysfunc(close(&dtstmatch.));

%if %EVAL(&nmatched.) = 0 %then %do;

DATA L1_&i;
	SET L1_TOT;
	drop winchester;
	L1 = 1;
	CODICE = "Cod_num_&i."; 
RUN;	
%if "&method" = "Sturges" %then %do;

proc freq data=keepdataset;
table &treat. /OUT = COUNT_TREAT_UNT (DROP = PERCENT);
run;
DATA _null_;
	SET COUNT_TREAT_UNT;
	CALL SYMPUT ('p', _n_);
	CALL SYMPUT (compress('tr'||_N_), trim(left(count)));
run;
%put NO MATCHED UNITS: EXECUTION TERMINATED;
%put ORIGINAL POPULATION: TREATED = &TR2. - UNTREATED = &tr1.;
%put MATCHED UNITS: TREATED = 0 - UNTREATED = 0;
%put L1 = 1;
%GOTO EXIT;
%end;

%end;
/************************************************************/

%ELSE %DO;

proc sql;
create table cod_dataset_&i as select
a.*,
b.matched
from cod_strata as a left join dataset_matched_&i. as b
on a.&id = b.&id
where matched = 1;
quit;

%L1_Nest (CODICE = &i,
ID1 = &id ,
TREAT1 = &treat,
COD_DATASET=cod_dataset_&i);
%END;


data L1_TOT;
set L1_TOT L1_&i;
drop winchester;
if codice = "" then delete;
run;

%if "&method" = "Sturges" %then %do;  /*if the binning option is Percent then skip*/
data L1_Stur;
set L1_TOT;
if substr(codice,9,1)^= "0" then codice = "Cod_num_stur";
run;
data cod_dataset_Stur;
	set cod_dataset_&i;
run;
data L1_TOT;
set L1_TOT;
if substr(codice,9,1)^= "0" then delete;
run;

%end;


%end;
%end;
/**********************************************/
/* MATCHING ONE-TO-N 			      */
/**********************************************/
%if &MATCH_TYPE. = N %then %do;
%do i=1 %to &maxcod.;
%if %eval(&numtot3.)>1 AND %eval(&numtot2.)=0 %then %do;
data cod_strata;
set rec_data;
cod_num_&i. = compress(&&cod_&i "");
run;
%end;
%if %eval(&numtot3.)=1 AND %eval(&numtot2.)=0 %then %do;
data cod_strata;
set rec_data;
cod_num_&i. = compress(&&cod_&i);
run;
%end;
%if %eval(&numtot2.)>0 %then %do;
data cod_strata;
set rec_data;
cod_num_&i. = compress(&&cod_&i||"");
run;
%end;
PROC FREQ DATA=cod_strata NOPRINT;
TABLES COD_NUM_&i.*&treat / OUT=FREQ_&i.(DROP=PERCENT);
run;
proc transpose data=FREQ_&i. out=FREQ_&i.(drop=_NAME_ _LABEL_) prefix=treat;
by COD_NUM_&i.;
id &treat.;
var COUNT;
run;
data FREQ_&i.;
set FREQ_&i.;
if treat0 = . then treat0=0;
if treat1 = . then treat1=0;
_NSIZE_ = min(treat0, treat1);
delta_freq = abs((treat0 / (treat0 + treat1)) - (treat1 / (treat0 +treat1)));
IF _NSIZE_=0 THEN DELETE;
run;

/*************** CHECK # OF MATCHED UNITS ********************/
%let dtstmatchN = %sysfunc(open(FREQ_&i.,i));
%let nmatchedN = %sysfunc(attrn(&dtstmatchN.,nobs));
%let dtsmN = %sysfunc(close(&dtstmatchN.));

%if &nmatchedN. = 0 %then %do;
DATA L1_&i;
	SET L1_TOT;
	drop winchester;
	L1 = 1;
	CODICE = "Cod_num_&i."; 
RUN;	
%if "&method" = "Sturges" %then %do;
proc freq data=keepdataset;
table &treat. /OUT = COUNT_TREAT_UNT (DROP = PERCENT);
run;
DATA _null_;
	SET COUNT_TREAT_UNT;
	CALL SYMPUT ('p', _n_);
	CALL SYMPUT (compress('tr'||_N_), trim(left(count)));
run;
%put NO MATCHED UNITS: EXECUTION TERMINATED;
%put ORIGINAL POPULATION: TREATED = &TR2. - UNTREATED = &tr1.;
%put MATCHED UNITS: TREATED = 0 - UNTREATED = 0;
%put L1 = 1;
%GOTO EXIT;
%end;

%end;
/************************************************************/

%else %do;
data freq_&i.;
set freq_&i.;
if treat0 = 0 OR treat1 = 0 then delete;
run;

proc means data=freq_&i. noprint;
output out=Mct_&i. (drop = _type_ _freq_) sum(treat1) = Mt sum(treat0) = Mc;
run;
data _NULL_;
set Mct_&i.;
call symput ('Mt', Mt);
call symput ('Mc', Mc);
run;
%let dsid8 = %sysfunc(open(FREQ_&i.,i));
%let numt_&i. = %sysfunc(attrn(&dsid8.,nobs));
%let rkt = %sysfunc(close(&dsid8.));
%if &&numt_&i. ne 0 %then %do;
data weight_&i;
set freq_&i.;
Wsc_&i. = (treat1 / treat0) * (&Mc. / &Mt.);
run;
proc sql;
create table cod_strata as select
a.*,
b.Wsc_&i.
from cod_strata as a left join weight_&i. as b
on a.cod_num_&i. = b.cod_num_&i.;
quit;
data cod_strata(drop=Wsc_&i.);
set cod_strata;
Wsc1_&i. = Wsc_&i.;
if &treat. = 1 then Wsc1_&i. = 1;
if Wsc_&i. = . then Wsc1_&i. = 0;
run;
%end;
data cod_dataset_&i;
set cod_strata;
matched = 1;
if Wsc1_&i. = 0 then delete;
run;


/**********************************************/

%L1_Nest_W (CODICE = &i,
ID1 = &id ,
TREAT1 = &treat,
COD_DATASET=cod_dataset_&i,
weight =  Wsc1_&i.);

/**********************************************/
%end;

data L1_TOT;
set L1_TOT L1_&i;
drop winchester;
if codice = "" then delete;
run;

%if "&method" = "Sturges" %then %do;  /*if the binning option is Percent then skip*/
data L1_Stur;
set L1_TOT;
if substr(codice,9,1)^= "0" then codice = "Cod_num_stur";
run;
data cod_dataset_Stur;
	set cod_dataset_&i;
run;
data L1_TOT;
set L1_TOT;
if substr(codice,9,1)^= "0" then delete;
run;
%end;

%end;
%end;
/******	MATCHING CLOSED*********/


proc sort data=L1_TOT; by L1; run;
data L1_appo;
	set L1_tot L1_Stur;
	if substr(codice,9,1)= "0" then delete;
run;


/*************** CHECK # OF MATCHED UNITS ********************/
proc MEANS data=L1_appo MEAN;
	OUTPUT OUT=CHECKMATCH (DROP = _TYPE_ _FREQ_) MEAN(L1)=M_L1;
RUN;
DATA _null_;
	SET CHECKMATCH;
	CALL SYMPUT ('MEANL1', trim(left(M_L1)));
run;

%IF &MEANL1. = 1 %THEN %DO;


proc freq data=keepdataset;
table &treat. /OUT = COUNT_TREAT_UNT (DROP = PERCENT);
run;
DATA _null_;
	SET COUNT_TREAT_UNT;
	CALL SYMPUT ('p', _n_);
	CALL SYMPUT (compress('tr'||_N_), trim(left(count)));
run;
%put NO MATCHED UNITS: EXECUTION TERMINATED;
%put ORIGINAL POPULATION: TREATED = &TR2. - UNTREATED = &TR1.;
%put MATCHED UNITS: TREATED = 0 - UNTREATED = 0;
%put L1 = 1;
%GOTO EXIT;
%END;
/************************************************************/



data L1_appo1 (keep = cod);
	set L1_appo;
	if substr(codice,9,1)= "0" then delete;
	cod=tranwrd(codice, "Cod_num_", "");
	if _N_>1 then stop;/**/
run;
data _NULL_;
set L1_appo1;
call symput ('numero', trim(left(cod)));
run;

%if &MATCH_TYPE. = N %then %do;
%if "&method" = "Sturges" %then %do;  /*if the binning option is Percent then skip*/
data cod_dataset_&numero.;	
	set cod_dataset_&numero.;
	rename Wsc1_1 = Wsc1_&numero.;
run;
%end;
proc sort data=cod_dataset_&numero. out=cod1_dataset_&numero.(keep = &ID matched Wsc1_&numero.); by &id; run;
proc sort data=keepdataset; by &ID; run;
data keepdataset;
update keepdataset cod1_dataset_&numero.;
by &ID;
run;
proc sort data=&lib..&dataset.; by &id; run;												
proc sort data=keepdataset(keep = &ID matched Wsc1_&numero.) out=tmp; by &ID; run;		
data tmp1;																				
	merge &lib..&dataset. tmp;															
	by &ID;																				
run;																					
data &lib..&dataset.;																	
	set tmp1;																			
run;																					
%end;

%else %do; /*ONLY IF METHOD = ONE*/
proc sort data=cod_dataset_&numero. out=cod1_dataset_&numero.(keep = &ID matched); by &id; run;
proc sort data=keepdataset; by &ID; run;
data keepdataset;
update keepdataset cod1_dataset_&numero.;
by &ID;
run;
proc sort data=&lib..&dataset.; by &ID; run;												
proc sort data=keepdataset(keep = &ID matched) out=tmp; by &ID; run;						
data tmp1;																				
	merge &lib..&dataset. tmp;															
	by &ID;																				
run;																					
data &lib..&dataset.;																	
	set tmp1;																			
run;																					

%end;

DATA &lib..L1_finale;
format L1_value $20.;
set L1_TOT L1_Stur;
%if "&method" ^= "Sturges" %then %do;  /*if the binning option is Percent then skip*/
if codice = compress("Cod_num_&numero") then do;
L1_value = "L1 Best";
output;
end;
%end;
if codice = "Cod_num_0" then do;
L1_value = "L1 Original";
output;
end;
if codice = "Cod_num_stur" then do;
L1_value = "L1 Sturges";
output;
end;
run;
proc sort data=&lib..L1_finale nodupkey; by codice; run;
data &lib..L1_finale ;
	set &lib..L1_finale ;
	drop codice;
run;
proc sort data=&lib..L1_finale out=&lib..L1_finale; by descending L1; run;


/****************************START L1-PLOT******************************************/
%if "&method" ^= "Sturges" %then %do;  /*if the binning option is Percent then do*/

proc freq data=keepdataset;
	table &treat / out=pippo1;
run;
data _null_;
	set pippo1;
	where &treat = 1;
	call symput (compress('num_treat'), trim(left(count)));
run;

%do i = 1 %to &maxcod;
proc freq data=Cod_dataset_&i;
	table &treat / out=pippo_&i;
run;

%let pluto1 = %sysfunc(open(pippo_&i,i));
%let numpluto_&i = %sysfunc(attrn(&pluto1.,nobs));
%let endpluto1 = %sysfunc(close(&pluto1.));

%if &&numpluto_&i. >0 %then %do;
data _null_;
	set pippo_&i;
	where &treat = 1;
	call symput (compress('n_match'||&i), trim(left(count)));
run;
%end;
%else %do;
%let n_match&i = 0;
%end;
%end;

data L1_tot1;
	set L1_tot;
	where codice ^= "Cod_num_stur";
	if substr(codice,9,1)= "0" then delete;
	cod=tranwrd(codice, "Cod_num_", "") / 1;
run;
proc sort data=L1_tot1; by cod; run;
data L1_tot1;
	set L1_tot1;
		Label Matchate = "Percentage of Matched Units";
	%do i = 1 %to &maxcod;
	if &&numpluto_&i.>0 then do;
	if _N_ = %eval(&i) then Matchate = %sysevalf(%eval(&&n_match&i) / %eval(&num_treat));
	end;
	%end;
	if Matchate=. then Matchate=0;
run;

proc freq data=Cod_dataset_stur;
	table &treat / out=pippo_stur;
run;
data _null_;
	set pippo_stur;
	where &treat = 1;
	call symput (compress('n_match_stur'), trim(left(count)));
run;

%let vref = %sysevalf( %eval(&n_match_stur) / %eval(&num_treat) );
data _null_;
	set L1_Stur;
	where codice = "Cod_num_stur";
	call symput (compress('href'), trim(left(L1)));
run;

proc sort data=L1_tot1; by descending L1; run;
ods listing;
goptions reset=all ; 
FILENAME file "&path_graph.\L1_&dataset..png";
goption device=PNG gsfname=file xpixels=1750 ypixels=900;
axis1 order = (0 to 1 by 0.1);
axis2 order = (0 to 1 by 0.1);
proc gplot data=L1_TOT1(where = (matchate^=0));
title "Matching Frontier - L1 vs Percentage of Treated Units Matched";
plot matchate*l1/ caxis=red 
vref = &vref cvref=red lvref=1
href = &href chref=red lhref=1
vaxis = axis1
haxis = axis2;
run;quit;
%end;
/****************************END L1-PLOT********************************/

%if "&report"  = "on" %then %do;
/**********************************************/
/* 		FINAL REPORT 		   			      */
/**********************************************/
ods _all_ close;
options nodate nonumber nocenter;
title "Macro CEM Matching summary";

ods html PATH="&path_graph"(URL=NONE) body="Report.html" style=default   /**/;
/*ods NOPROCTITLE;*/

proc freq data=keepdataset;
title "Original population";
table &treat;
run;

DATA ZP;
set &lib..L1_finale;
where L1_value = "L1 Original";
title "Multivariate L1 distance of the original population";
proc print;
run;

proc freq data=keepdataset;
title "Matched population";
where matched=1;
table &treat;
run;

%if "&method" = "Sturges" %then %do;  /*if the binning option is Percent then skip*/
DATA ZP1;
set &lib..L1_finale;
where L1_value = "L1 Sturges"; 
title "";
proc print;
run;
%end;
%else %do;  /*if the binning option is Percent then skip*/
DATA ZP1;
set &lib..L1_finale;
where L1_value = "L1 Best"; 
title "";
proc print;
run;
%end;

%if "&method" ^= "Sturges" %then %do;  /*if the binning option is Percent then skip
title "Choose your preferred solution in terms of L1 and Percentage of Treated Units Matched";*/
data L1_tot2 (rename=(matchate=Perc_Matched));
set L1_tot1 (drop = cod);
proc print;
run;quit;
%end;

ods html close;
%end;
/************************************************************/

%END;
%ELSE %DO;
%PUT Warning: records in the original dataset are not uniquely identified by the &ID variable. Remove duplicates or re-define your &ID variable!;
%END;

%EXIT: %mend;
%macro ff12(data=,newvarname=industry12,sic=sic,out=&data);
 
data &out;
    set &data;
    &newvarname = 12;
*1 NoDur  Consumer NonDurables -- Food, Tobacco, Textiles, Apparel, Leather, Toys;
         if &SIC ge   0100 and &SIC le  0999 	then &NEWVARNAME=1;
         else if &SIC ge   2000 and &SIC le 	2399 	then &NEWVARNAME=1;
         else if &SIC ge   2700 and &SIC le 	2749 	then &NEWVARNAME=1;
         else if &SIC ge   2770 and &SIC le 	2799 	then &NEWVARNAME=1;
         else if &SIC ge   3100 and &SIC le 	3199 	then &NEWVARNAME=1;
         else if &SIC ge   3940 and &SIC le 	3989 	then &NEWVARNAME=1;

 *2 Durbl  Consumer Durables -- Cars, TVs, Furniture, Household Appliances;
        else if &SIC ge   2500 and &SIC le   2519 	then &NEWVARNAME=2;
        else if &SIC ge   2590 and &SIC le 	2599 	then &NEWVARNAME=2;
        else if &SIC ge   3630 and &SIC le 	3659 	then &NEWVARNAME=2;
        else if &SIC ge   3710 and &SIC le 	3711 	then &NEWVARNAME=2;
        else if &SIC ge   3714 and &SIC le 	3714 	then &NEWVARNAME=2;
        else if &SIC ge   3716 and &SIC le 	3716 	then &NEWVARNAME=2;
        else if &SIC ge   3750 and &SIC le 	3751 	then &NEWVARNAME=2;
        else if &SIC ge   3792 and &SIC le 	3792 	then &NEWVARNAME=2;
        else if &SIC ge   3900 and &SIC le 	3939 	then &NEWVARNAME=2;
        else if &SIC ge   3990 and &SIC le 	3999 	then &NEWVARNAME=2;

* 3 Manuf  Manufacturing -- Machinery, Trucks, Planes, Off Furn, Paper, Com Printing;
        else if &SIC ge    2520 and &SIC le  2589 	then &NEWVARNAME=3;
        else if &SIC ge    2600 and &SIC le 	2699 	then &NEWVARNAME=3;
        else if &SIC ge    2750 and &SIC le 	2769 	then &NEWVARNAME=3;
        else if &SIC ge    3000 and &SIC le 	3099 	then &NEWVARNAME=3;
        else if &SIC ge    3200 and &SIC le 	3569 	then &NEWVARNAME=3;
        else if &SIC ge    3580 and &SIC le 	3629 	then &NEWVARNAME=3;
        else if &SIC ge    3700 and &SIC le 	3709 	then &NEWVARNAME=3;
        else if &SIC ge    3712 and &SIC le 	3713 	then &NEWVARNAME=3;
        else if &SIC ge    3715 and &SIC le 	3715 	then &NEWVARNAME=3;
        else if &SIC ge    3717 and &SIC le 	3749 	then &NEWVARNAME=3;
        else if &SIC ge    3752 and &SIC le 	3791 	then &NEWVARNAME=3;
        else if &SIC ge    3793 and &SIC le 	3799 	then &NEWVARNAME=3;
        else if &SIC ge    3830 and &SIC le 	3839 	then &NEWVARNAME=3;
        else if &SIC ge    3860 and &SIC le 	3899 	then &NEWVARNAME=3;

* 4 Enrgy  Oil, Gas, and Coal Extraction and Products;
        else if &SIC ge     1200 and &SIC le 	1399 	then &NEWVARNAME=4;
        else if &SIC ge     2900 and &SIC le 	2999 	then &NEWVARNAME=4;

* 5 Chems  Chemicals and Allied Products;
        else if &SIC ge     2800 and &SIC le 	2829 	then &NEWVARNAME=5;
        else if &SIC ge     2840 and &SIC le 	2899 	then &NEWVARNAME=5;

* 6 BusEq  Business Equipment -- Computers, Software, and Electronic Equipment;
        else if &SIC ge    3570 and &SIC le 	3579 	then &NEWVARNAME=6;
        else if &SIC ge    3660 and &SIC le 	3692 	then &NEWVARNAME=6;
        else if &SIC ge    3694 and &SIC le 	3699 	then &NEWVARNAME=6;
        else if &SIC ge    3810 and &SIC le 	3829 	then &NEWVARNAME=6;
        else if &SIC ge    7370 and &SIC le 	7379 	then &NEWVARNAME=6;

* 7 Telcm  Telephone and Television Transmission;
         else if &SIC ge    4800 and &SIC le 	4899 	then &NEWVARNAME=7;

* 8 Utils  Utilities;
         else if &SIC ge    4900 and &SIC le 	4949 	then &NEWVARNAME=8;

* 9 Shops  Wholesale, Retail, and Some Services (Laundries, Repair Shops);
        else if &SIC ge    5000 and &SIC le 	5999 	then &NEWVARNAME=9;
        else if &SIC ge    7200 and &SIC le 	7299 	then &NEWVARNAME=9;
        else if &SIC ge    7600 and &SIC le 	7699 	then &NEWVARNAME=9;

*10 Hlth   Healthcare, Medical Equipment, and Drugs;
         else if &SIC ge   2830 and &SIC le 	2839 	then &NEWVARNAME=10;
         else if &SIC ge   3693 and &SIC le 	3693 	then &NEWVARNAME=10;
         else if &SIC ge   3840 and &SIC le 	3859 	then &NEWVARNAME=10;
         else if &SIC ge   8000 and &SIC le 	8099 	then &NEWVARNAME=10;

*11 Money  Finance;
         else if &SIC ge   6000 and &SIC le 	6999 	then &NEWVARNAME=11;

*12 Other  Other -- Mines, Constr, BldMt, Trans, Hotels, Bus Serv, Entertainment;
		else if &SIC > .		and &NEWVARNAME = .	then &NEWVARNAME=12;

else &NEWVARNAME=0;

run;
%mend;
rsubmit;

%macro ff12(data=,newvarname=industry12,sic=sic,out=&data);
 
data &out;
    set &data;
    &newvarname = 12;
*1 NoDur  Consumer NonDurables -- Food, Tobacco, Textiles, Apparel, Leather, Toys;
         if &SIC ge   0100 and &SIC le  0999 	then &NEWVARNAME=1;
         else if &SIC ge   2000 and &SIC le 	2399 	then &NEWVARNAME=1;
         else if &SIC ge   2700 and &SIC le 	2749 	then &NEWVARNAME=1;
         else if &SIC ge   2770 and &SIC le 	2799 	then &NEWVARNAME=1;
         else if &SIC ge   3100 and &SIC le 	3199 	then &NEWVARNAME=1;
         else if &SIC ge   3940 and &SIC le 	3989 	then &NEWVARNAME=1;

 *2 Durbl  Consumer Durables -- Cars, TVs, Furniture, Household Appliances;
        else if &SIC ge   2500 and &SIC le   2519 	then &NEWVARNAME=2;
        else if &SIC ge   2590 and &SIC le 	2599 	then &NEWVARNAME=2;
        else if &SIC ge   3630 and &SIC le 	3659 	then &NEWVARNAME=2;
        else if &SIC ge   3710 and &SIC le 	3711 	then &NEWVARNAME=2;
        else if &SIC ge   3714 and &SIC le 	3714 	then &NEWVARNAME=2;
        else if &SIC ge   3716 and &SIC le 	3716 	then &NEWVARNAME=2;
        else if &SIC ge   3750 and &SIC le 	3751 	then &NEWVARNAME=2;
        else if &SIC ge   3792 and &SIC le 	3792 	then &NEWVARNAME=2;
        else if &SIC ge   3900 and &SIC le 	3939 	then &NEWVARNAME=2;
        else if &SIC ge   3990 and &SIC le 	3999 	then &NEWVARNAME=2;

* 3 Manuf  Manufacturing -- Machinery, Trucks, Planes, Off Furn, Paper, Com Printing;
        else if &SIC ge    2520 and &SIC le  2589 	then &NEWVARNAME=3;
        else if &SIC ge    2600 and &SIC le 	2699 	then &NEWVARNAME=3;
        else if &SIC ge    2750 and &SIC le 	2769 	then &NEWVARNAME=3;
        else if &SIC ge    3000 and &SIC le 	3099 	then &NEWVARNAME=3;
        else if &SIC ge    3200 and &SIC le 	3569 	then &NEWVARNAME=3;
        else if &SIC ge    3580 and &SIC le 	3629 	then &NEWVARNAME=3;
        else if &SIC ge    3700 and &SIC le 	3709 	then &NEWVARNAME=3;
        else if &SIC ge    3712 and &SIC le 	3713 	then &NEWVARNAME=3;
        else if &SIC ge    3715 and &SIC le 	3715 	then &NEWVARNAME=3;
        else if &SIC ge    3717 and &SIC le 	3749 	then &NEWVARNAME=3;
        else if &SIC ge    3752 and &SIC le 	3791 	then &NEWVARNAME=3;
        else if &SIC ge    3793 and &SIC le 	3799 	then &NEWVARNAME=3;
        else if &SIC ge    3830 and &SIC le 	3839 	then &NEWVARNAME=3;
        else if &SIC ge    3860 and &SIC le 	3899 	then &NEWVARNAME=3;

* 4 Enrgy  Oil, Gas, and Coal Extraction and Products;
        else if &SIC ge     1200 and &SIC le 	1399 	then &NEWVARNAME=4;
        else if &SIC ge     2900 and &SIC le 	2999 	then &NEWVARNAME=4;

* 5 Chems  Chemicals and Allied Products;
        else if &SIC ge     2800 and &SIC le 	2829 	then &NEWVARNAME=5;
        else if &SIC ge     2840 and &SIC le 	2899 	then &NEWVARNAME=5;

* 6 BusEq  Business Equipment -- Computers, Software, and Electronic Equipment;
        else if &SIC ge    3570 and &SIC le 	3579 	then &NEWVARNAME=6;
        else if &SIC ge    3660 and &SIC le 	3692 	then &NEWVARNAME=6;
        else if &SIC ge    3694 and &SIC le 	3699 	then &NEWVARNAME=6;
        else if &SIC ge    3810 and &SIC le 	3829 	then &NEWVARNAME=6;
        else if &SIC ge    7370 and &SIC le 	7379 	then &NEWVARNAME=6;

* 7 Telcm  Telephone and Television Transmission;
         else if &SIC ge    4800 and &SIC le 	4899 	then &NEWVARNAME=7;

* 8 Utils  Utilities;
         else if &SIC ge    4900 and &SIC le 	4949 	then &NEWVARNAME=8;

* 9 Shops  Wholesale, Retail, and Some Services (Laundries, Repair Shops);
        else if &SIC ge    5000 and &SIC le 	5999 	then &NEWVARNAME=9;
        else if &SIC ge    7200 and &SIC le 	7299 	then &NEWVARNAME=9;
        else if &SIC ge    7600 and &SIC le 	7699 	then &NEWVARNAME=9;

*10 Hlth   Healthcare, Medical Equipment, and Drugs;
         else if &SIC ge   2830 and &SIC le 	2839 	then &NEWVARNAME=10;
         else if &SIC ge   3693 and &SIC le 	3693 	then &NEWVARNAME=10;
         else if &SIC ge   3840 and &SIC le 	3859 	then &NEWVARNAME=10;
         else if &SIC ge   8000 and &SIC le 	8099 	then &NEWVARNAME=10;

*11 Money  Finance;
         else if &SIC ge   6000 and &SIC le 	6999 	then &NEWVARNAME=11;

*12 Other  Other -- Mines, Constr, BldMt, Trans, Hotels, Bus Serv, Entertainment;
		else if &SIC > .		and &NEWVARNAME = .	then &NEWVARNAME=12;

else &NEWVARNAME=0;

run;
%mend;
endrsubmit;





************;
*Prepare press coverage dataset;
************;
proc sql; create table pcDATA1 as select distinct
	a.*, b.gvkey, b.permco
	from mydir.PressCoverage a left join sp1500.sp1500 b
	on a.company_name eq b.coname
	where year(a.date) ge 2001;
quit;

*Reformat variables;
data pcDATA2; set pcDATA1;
gvkeyNUM = gvkey/1;
format date YYMMDDN8.;
drop gvkey;
rename gvkeyNUM=gvkey;
run;

*Address articles that are promoted from online to print;
data pc_online; set pcDATA2; where medium eq 'Online'; run;
data pc_print; set pcDATA2; where medium eq 'Print'; run;

proc sql; create table pc_print1 as select distinct
	a.*, b.date as dum
	from pc_print a left join pc_online b
	on a.company_name eq b.company_name and a.author_names eq b.author_names
	and intnx('day',b.date,0) <= a.date <= intnx('day',b.date,1);
quit;
data pc_print2; set pc_print1; if dum ne . then onl_promo=1; else onl_promo=0; drop dum; run;
proc sort data=pc_print2 out=pcDATA3 nodupkey; by company_name firm_article; run;

proc sql; create table pc_online1 as select distinct
	a.*, b.date as dum
	from pc_online a left join pc_print b
	on a.company_name eq b.company_name and a.author_names eq b.author_names
	and intnx('day',b.date,-1) <= a.date <= intnx('day',b.date,0);
quit;
data pc_online2; set pc_online1; where dum eq .; onl_promo=.; drop dum; run;

proc append base=pcDATA3 data=pc_online2; run;
proc sort data=pcDATA3; by company_name firm_article; run;

************;
*Prepare press release dataset;
************;
*Get character format gvkey and item numbers for each press release;
proc sql; create table pr1 as select distinct
	a.*, b.nitem, b.nitemno, b.group, b.gvkey as gvkeyCHAR, b.cik as cikCHAR, b.fdate, b.secadate, b.rdate
	from mydir.PressReleasesData a left join pr.pr_edgar_w8kitems b
	on a.fname eq b.fname
	where group ne 9;
quit;

data pr2; set pr1;
where (100 <= length <= 7000) and (0 < readability <= 30);
sdate = min(secadate,fdate);
if sdate-rdate gt 7 and month(intnx('day',rdate,0)+1) ne month(rdate) then prdate=sdate; else prdate=rdate;
format prdate YYMMDDN8.;
pryear=year(prdate);
drop fdate secadate rdate sdate;
run;

*Keep identifiers dataset then get controls;
data gc1; set pr2; keep gvkeyCHAR cikCHAR myfname prdate pryear group nitem; rename gvkeyCHAR=gvkey cikCHAR=cik; run;

rsubmit;
proc upload data=gc1; run;

*Get Compustat data;
*rsubmit;
proc sql; create table gc2 as select
	a.*, b.cusip, b.datadate, b.CEQ/1000 as CEQ, b.EMP/100 as EMP, log(1+(b.EMP/100)) as lnEMP, b.CSHR/100 as OWN, log(1+(b.CSHR/100)) as lnOWN
	from gc1 as a left join comp.funda as b
	on a.gvkey eq b.gvkey and b.datadate <= a.prdate and 0 <= a.pryear-year(b.datadate) <= 1
	where b.CEQ ne . and b.EMP ne . and b.CSHR ne .;
	quit;
*Keep Compustat data from most recently filed annual accounting data;
proc sort data=gc2; by gvkey myfname nitem descending datadate; run;
proc sort data=gc2 out=gc3 nodupkey; by gvkey myfname nitem; run;
*Get 8-digit cusip;
data gc4; length cusip8 $8.; set gc3; cusip8=substr(cusip,1,8); format cusip8 $8.; run;


*Get permnos from CRSP linking table to merge Compustat and CRSP data;
*rsubmit;
proc sql; create table gc5 as select 
	a.*, b.lpermno as permno, b.liid
    from gc4 as a left join crsp.ccmxpf_linktable
	(where=(linktype in ('LU', 'LC', 'LD', 'LF', 'LN', 'LO', 'LS', 'LX'))) as b
    on (a.gvkey = b.gvkey) 
	and (b.linkdt <= a.datadate or b.linkdt = .B) 
	and (a.datadate <= b.linkenddt or b.linkenddt = .E)
	where permno ne .
	order by gvkey, datadate;
 	quit;
proc sort data=gc5; by myfname nitem liid; run;
proc sort data=gc5 out=gc6 nodupkey; by myfname nitem; run;

*Get CRSP data;
*rsubmit;
data CRSPmonth; set crsp.msf (keep = permco permno date ret prc vol shrout); where (permno ne .) and (1998 <= year(date));
year=year(date);
mSIZE=abs(prc)*(shrout/1000000);
mTURN=vol/(shrout*10);
run;

*Size;
*Sum equity value of various share classes to get equity value of firm;
*rsubmit;
proc sql; create table size1 as select distinct
	permco, date, sum(mSIZE) as SIZE, log(1+sum(mSIZE)) as lnSIZE
	from CRSPmonth
	group by permco, date
	order by permco, date;
	quit;
proc sql;
	create table CRSPmonth1 as select
	a.*, b.SIZE, b.lnSIZE
	from CRSPmonth as a left join size1 as b
	on a.permco eq b.permco and a.date eq b.date
	order by a.permno, a.date;
quit;

*rsubmit;
proc expand data = CRSPmonth1 out = CRSPmonth2;
	convert ret = retvol12 / transformout=( movstd 12);
	convert mturn = mturn12 / transformout=( movave 12);
	by permno; id date; 
run;

*rsubmit;
proc sort data=CRSPmonth2; by permno date; run;
data CRSPmonth3; set CRSPmonth2; permno11=lag11(permno); run;
data CRSPmonth4; set CRSPmonth3;
if permno eq permno11 then RETVOL=retvol12*sqrt(12); else RETVOL=.;
if permno eq permno11 then TURN=mturn12; else TURN=.;
run;

*rsubmit;
proc sql; create table gc7 as select
	a.*, b.date, b.SIZE, b.lnSIZE, a.CEQ/b.SIZE as BTM, log(1+(a.CEQ/b.SIZE)) as lnBTM, b.RETVOL, b.TURN
	from gc6 as a left join CRSPmonth4 as b
	on a.permno eq b.permno and intnx('month',a.prdate,0) = intnx('month',b.date,1);
quit;


*Analyst following;
*rsubmit;
data ibes; set ibes.statsumu_epsus(keep = cusip fpi measure fpedats statpers usfirm numest);
where  (statpers < fpedats) and (fpedats-statpers <= 90) and (measure = 'EPS') and (usfirm eq 1) and (cusip ne '')
		and ((fpi = '6') or (fpi = '7') or (fpi = '8') or (fpi = '9'));
drop measure usfirm fpi fpedats;
run;
proc sort data=ibes; by cusip statpers; run;

*rsubmit;
proc sql; create table gc8 as select
	a.*, b.NUMEST as ANALYST
	from gc7 as a left join ibes as b
	on a.cusip8 eq b.cusip and intnx('month',a.prdate,0) = intnx('month',b.statpers,1)
	order by a.gvkey, a.myfname, a.nitem;
	quit;
data gc9; set gc8; if ANALYST eq . then ANALYST=0; lnANALYST=log(1+ANALYST); run;

*Quantity of Disclosure;
*rsubmit;
data forms; set wrdssec.wrds_forms (keep= cik fdate form fsize doccount);
where (1998 <= year(fdate)) and (form in ('8-K','8-K/A'));
year=year(fdate);
month=month(fdate);
drop form;
run;

*rsubmit;
proc sql; create table get8ks as select
	a.cik, a.myfname, a.nitem as nitem8k, a.prdate, b.fdate, b.fsize, b.doccount
	from gc9 as a left join forms as b
	on a.cik eq b.cik and intnx('month',a.prdate,-12) <= b.fdate <= intnx('month',a.prdate,0)
	order by a.cik, a.myfname, a.nitem, b.fdate;
	quit;

*rsubmit;
proc sql; create table get8ks1 as select distinct
	cik, myfname, nitem8k, prdate,
	count(fdate) as DISC8K, sum(fsize) as FSIZE8K, sum(doccount) as DOCS8K,
	log(1+count(fdate)) as lnDISC8K, log(1+sum(fsize)) as lnFSIZE8K, log(1+sum(doccount)) as lnDOCS8K
	from get8ks
	group by cik, myfname, nitem8k
	order by cik, myfname, nitem8k;
	quit;

*rsubmit;
proc sql; create table gc10 as select
	a.*, b.DISC8K, b.lnDISC8K/*, b.FSIZE8K, b.lnFSIZE8K, b.DOCS8K, b.lnDOCS8K*/
	from gc9 as a left join get8ks1 as b
	on a.myfname eq b.myfname and a.nitem eq b.nitem8k
	order by a.cik, a.myfname, a.nitem;
	quit;

*Quantity of Prior Press Coverage (following Twedt, 2016);

*rsubmit;
%macro append;
%do year= 2000 %to 2017;
data dj_equities;
set rpna.dj_equities_&year;
run;

%if &year=2000 %then %do;
data dj_equities_2000_2017;
set dj_equities;
run;
%end;

%else %do;
proc append base=dj_equities_2000_2017 data=dj_equities force;
run;
%end;

%end;
%mend;

%append;

*rsubmit;
data rpna; length cusip $8.; set dj_equities_2000_2017 (keep= isin rpna_date_utc entity_type news_type source rp_story_id relevance);
where (entity_type eq "COMP") and (news_type ne "PRESS-RELEASE") and (relevance ge 90);
cusip = substr(isin,3,8);
format cusip $8.;
run;
*Just keep one observation for each firm-day;
proc sort data=rpna out=rpna1 nodupkey; by cusip rpna_date_utc; run;

*rsubmit;
proc sql; create table getRPNA as select
	a.cusip8, a.myfname, a.nitem as nitem8k, a.prdate, b.rpna_date_utc
	from gc10 as a left join rpna1 as b
	on a.cusip8 eq b.cusip and intnx('month',a.prdate,-12) <= b.rpna_date_utc <= intnx('month',a.prdate,0)
	order by a.cusip8, a.myfname, a.nitem, b.rpna_date_utc;
	quit;

*rsubmit;
proc sql; create table getRPNA1 as select distinct
	cusip8, myfname, nitem8k, prdate, count(rpna_date_utc) as PRIORPC, log(1+count(rpna_date_utc)) as lnPRIORPC
	from getRPNA
	group by cusip8, myfname, nitem8k
	order by cusip8, myfname, nitem8k;
	quit;

*rsubmit;
proc sql; create table gc11 as select
	a.*, b.PRIORPC, b.lnPRIORPC
	from gc10 as a left join getRPNA1 as b
	on a.myfname eq b.myfname and a.nitem eq b.nitem8k
	order by a.gvkey, a.myfname, a.nitem;
	quit;


*Get market data around press release;
*rsubmit;
proc sql; create table prret2 as select
	a.*, b.date as anndate, b.RET
	from gc11 as a left join crsp.dsf as b
	on a.permno eq b.permno
	and a.prdate <= b.date <= intnx('day',a.prdate,120)
	order by a.myfname, a.nitem, b.date;
quit;

*Get intraperiod timeliness around press release;
*rsubmit;
data prret3; set prret2; myfname_nitem= strip(myfname) || strip(nitem); run;
proc sort data=prret3; by myfname_nitem date; run;
data prret4; set prret3; count + 1; by myfname_nitem; if first.myfname_nitem then count = 0; run;
data prret5; set prret4; where count le 5; run;

*rsubmit;
proc sql; create table prret6 as select a.*, b.vwretd, 1+a.ret as ret1, 1+b.vwretd as vwretd1 from prret5 as a left join crsp.dsi as b on a.anndate eq b.date; quit;
proc sort data=prret6; by myfname group anndate; run;

*Calculate intraperiod timeliness (0,6);
*rsubmit;
proc expand data = prret6 out = prret6_0; by myfname;
convert ret1 = BHret1 / transformout=( movprod 6);
convert vwretd1 = BHvwretd1 / transformout=( movprod 6);
run;
data prret6_1; set prret6_0; where count lt 5; abBHret=BHret1-BHvwretd1; run;
data prret6_last; set prret6_0; where count eq 5; TOTabBHret=BHret1-BHvwretd1; run;

*rsubmit;
proc sql; create table prret6_2 as select distinct
	myfname, group, sum(abBHret) as abBHret04
	from prret6_1 
	group by myfname, group
	order by myfname, group;
	quit;

*rsubmit;
proc sql; create table prret6_3 as select
	a.*, b.TOTabBHret as CAR05, (a.abBHret04/b.TOTabBHret)+0.5 as IPT05
	from prret6_2 as a left join prret6_last as b
	on a.myfname eq b.myfname and a.group eq b.group
	order by a.myfname, a.group;
	quit;

*rsubmit;
proc sql; create table gc11_IPT as select
	a.*, b.CAR05, b.IPT05
	from gc11 as a left join prret6_3 as b
	on a.myfname eq b.myfname and a.group eq b.group
	order by a.gvkey, a.myfname, a.group;
	quit;


*Get abnormal returns during and after press release;
*rsubmit;
data ea; set prret4; where count le 75; run;


*Get value weighted market returns;
*rsubmit;
proc sql; create table ea1 as select
	a.*, b.vwretd
	from ea as a left join crsp.dsi as b
	on a.anndate eq b.date;
	quit;
proc sort data=ea1; by myfname_nitem prdate anndate; run;

*Split into EA and post-EA;
*rsubmit;
data duringEA; set ea1; where count le 2; run;
data afterEA; set ea1; where count gt 2; run;

*Calculate abnormal returns, noise during EA, and turnover during EA;
proc sql; create table duringEA1 as select distinct
	myfname, group, prdate, exp(sum(log(1+ret))) - exp(sum(log(1+vwretd))) as retEA, count(ret) as nEA
	from duringEA 
	group by myfname, group, prdate
	order by myfname, group, prdate;
	quit;

proc sql; create table afterEA1 as select distinct
	myfname, group, prdate, exp(sum(log(1+ret))) - exp(sum(log(1+vwretd))) as retPOST, count(ret) as nPOST
	from afterEA 
	group by myfname, group, prdate
	order by myfname, group, prdate;
	quit;

*Merge to other data;
*rsubmit;
proc sql;
	create table gc11_EA as select
	a.*, b.retEA*100 as retEA
	from gc11_IPT as a left join duringEA1 as b
	on a.myfname eq b.myfname and a.group eq b.group;

	create table gc12 as select
	a.*, b.retPOST*100 as retPOST
	from gc11_EA as a left join afterEA1 as b
	on a.myfname eq b.myfname and a.group eq b.group
	order by a.gvkey, a.myfname, a.group;
quit;


*  Abnormal volume and adverse selection (abnormal bid-ask spread and MIA);
*rsubmit;
proc upload data=mydir.miadata2006to2015; run;

*rsubmit;
data aspr; set gc12; keep permno prdate; run;

proc sql; 
	*Volume and spread;
	create table aspr0 as select
	a.*, b.date, b.VOL, log(1+((b.vol/1000)/b.shrout)) as volume, b.BID, b.ASK, (b.ask-b.bid)/((b.ask+b.bid)/2) as spread
	from aspr as a left join crsp.dsf as b
	on a.permno eq b.permno
	and intnx('day',a.prdate,-100) <= b.date <= intnx('day',a.prdate,6)
	where b.ask-b.bid ge 0;
	
	*MIA;
	create table aspr1 as select
	a.*, b.mia
	from aspr0 as a left join miadata2006to2015 as b
	on a.permno eq b.permno and a.date eq b.date;
quit;

*rsubmit;
*Identify days in (t-60,t+2);
data aspr2; set aspr1; if prdate >= date then keep=1; else keep=0; run;
proc sort data=aspr2; by permno prdate date; run;
data aspr3; set aspr2; lagkeep=lag(keep); lagkeep2=lag2(keep); run;
data aspr4; set aspr3; where lagkeep=1 or lagkeep2=1 or keep=1; drop keep lagkeep lagkeep2; run;

*rsubmit;
*Only keep days in (t-60,t+2);
proc sort data=aspr4; by permno prdate descending date; run;
data aspr5; set aspr4; firmdate = permno + prdate/1000000; run;
data aspr6; set aspr5; count + 1; by firmdate; if first.firmdate then count = -2; run;
data aspr7; set aspr6; where (count le 60);  run;

*Split into EA and pre-EA;
*rsubmit;
data duringEA; set aspr7; where count le 0; run;
data beforeEA; set aspr7; where count gt 0; run;

*Calculate abnormal returns, noise during EA, and turnover during EA;
proc sql; create table duringEA1 as select distinct
	permno, prdate, mean(volume) as during_volume, mean(spread) as during_spread, mean(mia) as during_mia, count(spread) as nEA
	from duringEA 
	group by permno, prdate
	order by permno, prdate;
	quit;
data duringEA2; set duringEA1; if nEA lt 3 then during_volume=.; if nEA lt 3 then during_spread=.; run;

proc sql; create table beforeEA1 as select distinct
	permno, prdate, mean(volume) as pre_volume, mean(spread) as pre_spread, mean(mia) as pre_mia, count(spread) as nPRE
	from beforeEA 
	group by permno, prdate
	order by permno, prdate;
	quit;
data beforeEA2; set beforeEA1; if nPRE lt 60 then pre_volume=.; if nPRE lt 60 then pre_spread=.; run;

*Merge to other data;
*rsubmit;
proc sql;
	create table gc12_pre1 as select
	a.*, b.during_volume, b.during_spread, b.during_mia
	from gc12 as a left join duringEA2 as b
	on a.permno eq b.permno and a.prdate eq b.prdate;

	create table gc12_pre2 as select
	a.*, b.pre_volume, b.pre_spread, b.pre_mia
	from gc12_pre1 as a left join beforeEA2 as b
	on a.permno eq b.permno and a.prdate eq b.prdate;
quit;

*Calculate abnormal variables;
*rsubmit;
data gc13; set gc12_pre2;
ABvolume=during_volume-pre_volume;
ABspread=during_spread-pre_spread;
ABmia=during_mia-pre_mia;
rename during_mia=mia;
run;




*************  Analyst-based surprise;

*Get earnings and lag price from Compustat quarterly data;
*rsubmit;
proc sql; create table gc13_earn as select
	a.*, b.fyearq as fyearq_q, b.fqtr as fqtr_q, b.cusip, b.datadate as datadate_q, b.rdq, b.EPSPXQ, b.PRCCQ
	from gc13 as a left join comp.fundq as b
	on a.gvkey eq b.gvkey and b.datadate <= a.prdate and 0 <= a.pryear-year(b.datadate) <= 1;
	quit;

*Calculate dEPS;
*rsubmit;
proc sql; create table gc13_earn_wdEPS as select
	a.*, b.EPSPXQ as lEPSPXQ
	from gc13_earn as a left join comp.fundq as b
	on a.gvkey eq b.gvkey and a.fyearq_q-1 eq b.fyearq and a.fqtr_q eq b.fqtr;
quit;

*Get lag price;
*rsubmit;
proc sort data=gc13_earn_wdEPS; by gvkey myfname nitem datadate_q; run;
data gc13_earn1; set gc13_earn_wdEPS; lagmyfname=lag(myfname); lagnitem=lag(nitem); lPRCCQ=lag(PRCCQ); run;
data gc13_earn2; set gc13_earn1; if myfname ne lagmyfname or nitem ne lagnitem then lPRCCQ=.; drop lagmyfname lagnitem PRCCQ; run;

*Keep Compustat data from most recently filed annual accounting data;
*rsubmit;
proc sort data=gc13_earn2; by gvkey myfname nitem descending datadate_q; run;
proc sort data=gc13_earn2 out=gc13_earn3 nodupkey; by gvkey myfname nitem; run;


*Get IBES data for calculating surprise following Livnat and Mendenhall (2006, JAR);
*rsubmit;
data ibes; set ibes.detu_epsus(keep = ticker cusip anndats fpi measure usfirm fpedats value analys);
where  (measure = 'EPS') and (usfirm eq 1)
		and ((fpi = '6') or (fpi = '7') or (fpi = '8') or (fpi = '9'));
rename fpedats=fpe_date anndats=fcast_date value=fcast analys=analyst;
drop measure usfirm fpi;
run;
*Keep the most recent forecast for each analyst;
proc sort data=ibes; by ticker fpe_date analyst descending fcast_date; run;
proc sort data=ibes nodupkey; by ticker fpe_date analyst; run;

*Keep only forecasts issued in 90 days prior to end of forecast period;
*rsubmit;
data ibes1; set ibes; where 1 <= fpe_date-fcast_date <= 150; run;

*Calculate median forecast;
proc sql; create table ibes2 as select distinct
	ticker, fpe_date, median(fcast) as fcast
	from ibes1 group by cusip, fpe_date order by ticker, fpe_date;
	quit;

*** Merge back to other data;
*Get ibes ticker;
*rsubmit;
proc upload data=iclink.iclink out=iclink; run;
proc sql;
	create table gc13_earn3_wID as 
	select a.*, b.ticker as ibesticker, b.score
	from gc13_earn3 as a left join iclink as b
	on a.permno = b.permno;
quit;
proc sort data=gc13_earn3_wID; by gvkey myfname nitem score; run;
proc sort data=gc13_earn3_wID out=gc13_earn3_wID1 nodupkey; by gvkey myfname nitem; run;

*Calculate SURPRISE;
*rsubmit;
proc sql; create table gc14 as select
	a.*, b.fcast, ((a.EPSPXQ-b.fcast)/(a.lPRCCQ))*100 as SURP, ((a.EPSPXQ-a.lEPSPXQ)/(a.lPRCCQ))*100 as dEPS
	from gc13_earn3_wID1 as a left join ibes2 as b
	on a.ibesticker eq b.ticker
	and a.datadate_q eq b.fpe_date
	order by a.myfname, a.nitem;
quit;


*Get number of earnings announcements on same day;
*rsubmit;
data compq; set comp.fundq (keep = gvkey datadate rdq indfmt datafmt popsrc consol PRCCQ CSHOQ);
where (1998 <= year(datadate) <= 2015) and (indfmt='INDL') and (datafmt='STD') and (popsrc='D') and (consol='C') and (rdq ne .) and (PRCCQ ne .) and (CSHOQ ne .);
SIZE=(PRCCQ*CSHOQ)/1000;
drop indfmt datafmt popsrc consol;
run;

*rsubmit;
proc sort data=compq; by rdq; run;
proc sql; create table compq1 as select distinct
	rdq, count(rdq) as BUSY1, sum(SIZE) as BUSY2
	from compq group by rdq order by rdq;
	quit;

*rsubmit;
proc sql; create table numberEAs as select
	a.*, b.rdq, b.BUSY1, b.BUSY2
	from gc14 as a left join compq1 as b
	on intnx('day',a.prdate,-1) <= b.rdq <= intnx('day',a.prdate,1)
	order by a.myfname, a.group, b.rdq;
quit;

*rsubmit;
proc sql; create table numberEAs1 as select distinct
	myfname, group, sum(BUSY1) as BUSY1, sum(BUSY2) as BUSY2
	from numberEAs
	group by myfname, group
	order by myfname, group;
	quit;

*rsubmit;
proc sql; create table gc15rdq as select
	a.*, b.BUSY1, b.BUSY2, log(1+b.BUSY1) as lnBUSY1, log(1+b.BUSY2) as lnBUSY2
	from gc14 as a left join numberEAs1 as b
	on a.myfname eq b.myfname and a.group eq b.group
	order by a.myfname, a.group;
quit;

*Get closest rdq;
*rsubmit;
proc sql; create table gc15rdq1 as select
	a.*, b.rdq as rdqMATCH, abs(a.prdate-b.rdq) as CLOSErdq
	from gc15rdq as a left join comp.fundq as b
	on a.gvkey eq b.gvkey and intnx('month',a.prdate,-4) <= b.datadate <= intnx('month',a.prdate,4);
	quit;

*Keep Compustat data from most recently filed annual accounting data;
*rsubmit;
proc sort data=gc15rdq1; by gvkey myfname nitem CLOSErdq; run;
proc sort data=gc15rdq1 out=gc15 nodupkey; by gvkey myfname nitem; run;

*Download WRDS data;
*rsubmit;
proc download data=gc15 out=mydir.wrds_gc15_20170912; run;
endrsubmit;

*Get Institutional Ownership;
proc sql; create table gc16 as select
	a.*, b.INSTOWN
	from mydir.wrds_gc15_20170621 as a left join instown.instown_1980to2015 as b
	on a.permno eq b.permno and intnx('month',a.prdate,-3) <= b.rdate <= intnx('month',a.prdate,0)
	order by a.gvkey, a.myfname, a.nitem;
	quit;

*Get S&P 1500 indicator;
rsubmit;
proc upload data=gc16; run;
proc sql; create table gc17 as select
	a.*, b.SPMIM, abs(a.prdate-b.datadate) as closestSPMIM
	from gc16 as a left join comp.secm as b
	on a.gvkey eq b.gvkey and intnx('month',a.prdate,-2) <= b.datadate <= intnx('month',a.prdate,2)
	where SPMIM ne '';
	quit;
proc sort data=gc17; by myfname nitem closestSPMIM; run;
proc sort data=gc17 nodupkey; by myfname nitem; run;


* Get prior month stock return volatility for matching procedure;
*rsubmit;
proc sql; create table matchRV0 as select
	a.permno, a.myfname, a.nitem as nitem8k, a.prdate, b.date, b.ret
	from gc17 as a left join crsp.dsf as b
	on a.permno eq b.permno and intnx('day',a.prdate,-31) <= b.date <= intnx('day',a.prdate,-2)
	order by a.permno, a.myfname, a.nitem, b.date;
	quit;

*rsubmit;
proc sql; create table matchRV1 as select distinct
	permno, myfname, nitem8k, prdate, std(ret) as PRIORmRV
	from matchRV0
	group by permno, myfname, nitem8k
	order by permno, myfname, nitem8k;
	quit;

*rsubmit;
proc sql; create table gc19 as select
	a.*, b.PRIORmRV
	from gc17 as a left join matchRV1 as b
	on a.myfname eq b.myfname and a.nitem eq b.nitem8k
	order by a.gvkey, a.myfname, a.nitem;
	quit;

*rsubmit;
proc sort data=gc19 nodupkey; by myfname nitem; run;
proc download data=gc19 out=mydir.gc19_20171017; run;
endrsubmit;



* Get proportion of retail trading volume;

data retail; set mydir.gc19_20171017; keep permno myfname nitem prdate; run;

proc sql; create table retail1 as select
	a.*, b.date as taqdt,
	log(1+(((b.rbivol+b.rsivol)/1000)/b.shrout)) as rvolume, log(1+(((b.othvol)/1000)/b.shrout)) as ovolume, (b.rbivol+b.rsivol)/b.totvol as rpct
	from retail as a left join mytaq.TAQretail_060708121314 as b
	on a.permno eq b.permno
	and intnx('day',a.prdate,-100) <= b.date <= intnx('day',a.prdate,6)
	order by a.myfname, a.nitem, b.date;
quit;

*Identify days in (t-60,t+2);
data retail2; set retail1; if prdate >= taqdt then keep=1; else keep=0; run;
proc sort data=retail2; by myfname nitem prdate taqdt; run;
data retail3; set retail2; lagkeep=lag(keep); lagkeep2=lag2(keep); run;
data retail4; set retail3; where lagkeep=1 or lagkeep2=1 or keep=1; drop keep lagkeep lagkeep2; run;

*Only keep days in (t-60,t+2);
proc sort data=retail4; by myfname nitem descending taqdt; run;
data retail5; set retail4; myfnamenitem=strip(myfname) || strip(nitem); run;
data retail6; set retail5; count + 1; by myfnamenitem; if first.myfnamenitem then count = -2; run;
data retail7; set retail6; where (count le 60);  run;

*Split into EA and pre-EA;
data dretail; set retail7; where count le 0; run;
data bretail; set retail7; where count gt 0; run;

*Calculate abnormal returns, noise during EA, and turnover during EA;
proc sql; create table dretail1 as select distinct
	myfname, nitem, prdate, mean(rvolume) as during_rvolume, mean(ovolume) as during_ovolume, mean(rpct) as rpct
	from dretail 
	group by myfname, nitem, prdate
	order by myfname, nitem, prdate;
	quit;
proc sql; create table bretail1 as select distinct
	myfname, nitem, prdate, mean(rvolume) as pre_rvolume, mean(ovolume) as pre_ovolume
	from bretail 
	group by myfname, nitem, prdate
	order by myfname, nitem, prdate;
	quit;

*Merge to other data;
proc sql;
	create table gc19_pre1 as select
	a.*, b.during_rvolume, b.during_ovolume, b.rpct
	from mydir.gc19_20171017 as a left join dretail1 as b
	on a.myfname eq b.myfname and a.nitem eq b.nitem and a.prdate eq b.prdate;

	create table gc19_pre2 as select
	a.*, b.pre_rvolume, b.pre_ovolume
	from gc19_pre1 as a left join bretail1 as b
	on a.myfname eq b.myfname and a.nitem eq b.nitem and a.prdate eq b.prdate;
quit;

*Calculate abnormal variables;
data gc19; set gc19_pre2;
ABrvolume=during_rvolume-pre_rvolume;
ABovolume=during_ovolume-pre_ovolume;
run;


* Get earnings announcement (DJ and WSJ) coverage;
data rpna; length cusip $8.; set rpna.rp_dj (keep= isin TIMESTAMP_UTC news_type source relevance);
where (ISIN ne "") and (news_type ne "PRESS-RELEASE") and (SOURCE in ("B5569E","AA6E89")) and (relevance ge 90);
DATE = DATEPART(TIMESTAMP_UTC); format DATE YYMMDDN.;
cusip = substr(isin,3,8);
format cusip $8.;
drop TIMESTAMP_UTC relevance;
run;

*Count articles on day t;
proc sql; create table rpna_d as select distinct
	cusip, date, source, news_type, count(date) as EAPC
	from rpna
	group by cusip, date, source;
	quit;

data rpna_dj; set rpna_d; where source eq "B5569E"; run;
data rpna_wsj; set rpna_d; where source eq "AA6E89"; run;

*Count articles in (-3,+3);
proc sql; create table rpna_d1 as select
	a.cusip, a.date, b.EAPC
	from rpna_d as a left join rpna_dj as b
	on a.cusip eq b.cusip and intnx('day',a.date,-3) <= b.date <= intnx('day',a.date,3)
	order by a.cusip, a.date, b.date;
	quit;
proc sql; create table rpna_d2 as select
	a.cusip, a.date, a.EAPC, b.EAPC as EAPC_wsj
	from rpna_d1 as a left join rpna_wsj as b
	on a.cusip eq b.cusip and intnx('day',a.date,-3) <= b.date <= intnx('day',a.date,3)
	order by a.cusip, a.date, b.date;
	quit;
proc sql; create table rpna_d3 as select distinct
	cusip, date, count(EAPC) as EAPC, count(EAPC_wsj) as EAPC_wsj
	from rpna_d2
	group by cusip, date;
	quit;

*Merge to other data;
proc sql; create table gc20 as select
	a.*, b.EAPC, log(1+EAPC) as lnEAPC, b.EAPC_wsj, log(1+EAPC_wsj) as lnEAPC_wsj
	from gc19 as a left join rpna_d3 as b
	on a.cusip8 eq b.cusip and a.prdate eq b.date
	order by a.gvkey, a.myfname, a.nitem;
	quit;


***************************************:
* Merge to wsj articles;
***************************************;

proc sql; create table mydir.gc20_20181115 as select distinct
	a.*, b.fname, b.exname, b.readability*(-1) as READABLE, b.hsmix as HARDINFO, b.specificity as SPECIFIC
	from gc20 a left join pr2 b
	on a.myfname eq b.myfname and a.nitem eq b.nitem
	where a.prdate >= intnx('day','01JUL2003'd,0);
quit;

****************************;
*Get industry classification;
****************************;

data forup; set mydir.gc20_20181115; keep myfname nitem gvkey permno prdate; run;
rsubmit; 
proc upload data=forup; run;

*Get Compustat SIC code;
*rsubmit;
proc sql; create table wsic1 as select 
	a.*, b.sich
    from forup as a left join comp.funda as b
    on (a.gvkey = b.gvkey) and year(a.prdate) eq b.fyear;
 	quit;
proc sort data=wsic1 out=wsic2 nodupkey; by myfname nitem gvkey prdate sich; run;

*Get CRSP SIC code;
*rsubmit;
proc sql; create table wsic3 as select 
	a.*, b.siccd
    from wsic2 as a left join crsp.msenames as b
    on (a.permno = b.permno) and (b.namedt <= a.prdate) and (a.prdate <= b.nameendt);
 	quit;

*Use Compustat sic code if available;
*rsubmit;
data wsic4; set wsic3;
if sich ne . then sic=sich; else sic=siccd;
drop sich siccd;
run;

*rsubmit;
proc download data=wsic4; run;
endrsubmit;

*Assign to FF-12 industries;
%ff12(data=wsic4,newvarname=industry12,sic=sic,out=wsic5);

*Merge to main data;
proc sql; create table gc21 as select distinct
	a.*, b.sic, b.industry12
	from mydir.gc20_20181115 a left join wsic5 b
	on a.myfname eq b.myfname and a.nitem eq b.nitem where a.prdate = b.prdate;
quit;


*Create pseudo-firms based on size and btm;
proc rank data=gc21 out=pseudo1 groups=10; var SIZE; ranks rSIZE; run;
proc rank data=pseudo1 out=pseudo2 groups=10; var BTM; ranks rBTM; run;
data mydir.gc21_20200212; set pseudo2; pseudo=cat(rSIZE,rBTM); drop rSIZE rBTM; run;


*Get non-WSJ (i.e., WEB edition aka WE) earnings coverage;
rsubmit;
%macro append;
%do year= 2004 %to 2015;
data rp_equities; set rpna.rp_equities_&year (keep= isin rpna_date_utc entity_type news_type source rp_story_id product_key relevance);
where (ISIN ne "") and (entity_type eq "COMP") and (news_type eq "FULL-ARTICLE") and (relevance eq 100)
and source in ('B5569E','751371','AA1167','22AC8B','C325FC','6DFE43','208421','18A55F','C26100','1E5E35','39C1F7','B6ACEE','FD0B00','C0FD19');
CUSIP8 = substr(ISIN,3,8);
run;

%if &year=2004 %then %do;
data rp_equities_2004_2015; set rp_equities;
run;
%end;

%else %do;
proc append base=rp_equities_2004_2015 data=rp_equities force;
run;
%end;

%end;
%mend;
%append;

*rsubmit;
*Count articles on day t;
proc sql; create table raven as select distinct
	cusip8, rpna_date_utc, source, count(rpna_date_utc) as narts
	from rp_equities_2004_2015
	group by cusip8, rpna_date_utc, source;
	quit;

*Count articles in (-1,+1);
proc sql; 
	create table raven1 as select
	a.*, b.rpna_date_utc as lagrpna_date_utc, b.narts as lagnarts
	from raven as a left join raven as b
	on a.cusip8 eq b.cusip8 and a.source eq b.source and intnx('day',a.rpna_date_utc,-1) = b.rpna_date_utc;

	create table raven2 as select
	a.*, b.rpna_date_utc as leadrpna_date_utc, b.narts as leadnarts
	from raven1 as a left join raven1 as b
	on a.cusip8 eq b.cusip8 and a.source eq b.source and intnx('day',a.rpna_date_utc,1) = b.rpna_date_utc;
quit;
data raven3; set raven2; if lagnarts eq . then lagnarts=0; if leadnarts eq . then leadnarts=0; run;
data raven4; set raven3; newnarts=narts+lagnarts+leadnarts; drop narts lagnarts leadnarts lagrpna_date_utc leadrpna_date_utc; run;

*rsubmit;
*Get DJ and WE coverage separately;
data ravenDJ; set raven4; where source in ('B5569E'); run;
data ravenWE; set raven4; where source in ('751371','AA1167','22AC8B','C325FC','6DFE43','208421','18A55F','C26100','1E5E35','39C1F7','B6ACEE','FD0B00','C0FD19'); run;



**** Get media coverage of each FF 12 industry ****;
*rsubmit;
*Count articles on day t;
proc sql; create table raven5 as select distinct
	cusip8, rpna_date_utc, count(newnarts) as narts
	from ravenWE
	group by cusip8, rpna_date_utc;
	quit;

*rsubmit;
*Get GVKEY and SICH;
proc sql; create table raven6 as select
	a.*, b.gvkey, b.datadate, b.sich
	from raven5 as a left join comp.funda as b
	on a.cusip8 eq substr(b.cusip,1,8) and b.datadate <= a.rpna_date_utc and 0 <= year(a.rpna_date_utc)-year(b.datadate) <= 1;
	quit;
*Keep Compustat data from most recently filed annual accounting data;
proc sort data=raven6; by cusip8 rpna_date_utc descending datadate; run;
proc sort data=raven6 out=raven7 nodupkey; by cusip8 rpna_date_utc; run;

*rsubmit;
*Get PERMNO;
proc sql; create table raven8 as select 
	a.*, b.lpermno as permno, b.liid
    from raven7 as a left join crsp.ccmxpf_linktable
	(where=(linktype in ('LU', 'LC', 'LD', 'LF', 'LN', 'LO', 'LS', 'LX'))) as b
    on (a.gvkey = b.gvkey) 
	and (b.linkdt <= a.datadate or b.linkdt = .B) 
	and (a.datadate <= b.linkenddt or b.linkenddt = .E)
	where permno ne .
	order by cusip8, rpna_date_utc;
 	quit;
proc sort data=raven8; by cusip8 rpna_date_utc liid; run;
proc sort data=raven8 out=raven9 nodupkey; by cusip8 rpna_date_utc; run;
*Get CRSP SIC code;
proc sql; create table raven10 as select 
	a.*, b.siccd
    from raven9 as a left join crsp.msenames as b
    on (a.permno = b.permno) and (b.namedt <= a.rpna_date_utc) and (a.rpna_date_utc <= b.nameendt);
 	quit;

*Use Compustat sic code if available;
*rsubmit;
data raven11; set raven10;
if sich ne . then sic=sich; else sic=siccd;
drop sich siccd;
run;
*Assign to FF-12 industries;
%ff12(data=raven11,newvarname=industry12,sic=sic,out=raven12);

*rsubmit;
*Count articles;
proc sql; create table ravenIND as select distinct
	industry12, rpna_date_utc, count(narts) as newnarts_ind12
	from raven12
	group by industry12, rpna_date_utc;
	quit;

*rsubmit;
proc download data=ravenDJ; run;
proc download data=ravenWE; run;
proc download data=ravenIND; run;
endrsubmit;


*Merge to other data;
proc sql; 
	create table othcov1 as select
	a.*, b.newnarts as nartsDJ
	from mydir.gc21_20200212 as a left join ravenDJ as b
	on a.cusip8 eq b.cusip8 and a.prdate = b.rpna_date_utc;
	
	create table mydir.gc21_20200327 as select
	a.*, b.newnarts as nartsWE
	from othcov1 as a left join ravenWE as b
	on a.cusip8 eq b.cusip8 and a.prdate = b.rpna_date_utc
	order by a.gvkey, a.myfname, a.nitem;
quit;

proc sql; 
	create table mydir.gc21_20200515 as select
	a.*, b.newnarts_ind12 as nartsIND, log(1+b.newnarts_ind12) as ln_nartsIND
	from mydir.gc21_20200327 as a left join ravenIND as b
	on a.industry12 eq b.industry12 and a.prdate = b.rpna_date_utc
	order by a.gvkey, a.myfname, a.nitem;
quit;




*Merge in textual analysis variables for comparing earnings release and WSJ article, e.g., word and tone diff, number of words;

proc sql; create table pcpr0 as select
	a.*, b.PCdate-a.PRdate as DATE_DIFF, abs(b.PCdate-a.PRdate) as absDATE_DIFF, b.PRtotal_words as PRLENGTH, b.PCtotal_words as PCLENGTH,
	b.company_name, b.pc_fname, b.pcdate, year(b.pcdate) as pcyear, b.PCnegativity as PCTONE, b.PRnegativity as PRTONE,
	abs(b.PCnegativity-b.PRnegativity) as TONEDIFF1, b.similarity*100 as DISS
	from mydir.gc21_20200515 as a left join ta.AnalyzedMatches as b
	on a.myfname eq b.pr_fname and intnx('day',b.pcdate,-1) <= a.prdate <= b.pcdate
	order by a.gvkey, a.myfname;
	quit;

proc sql; create table pcpr1 as select distinct
	a.*, b.medium, b.position, b.onl_promo
	from pcpr0 a left join pcDATA3 b
	on a.pc_fname eq b.fname;
quit;




***************************************************************************;
************************ Baseline (for Table 3) ***************************;
***************************************************************************;

*Clean data for baseline;
data base2; set pcpr1;
where (CLOSErdq le 4) and (SPMIM eq '10') and (group in (1,2,5,7,8))
	and (lnSIZE ne .) and (lnBTM ne .) and (RETVOL ne .) and (TURN ne .) and (lnEMP ne .) and (lnOWN ne .)
	and (lnANALYST ne .) and (lnDISC8K gt 0) and (retEA ne .) /*and (INSTOWN ne .) and (lnPRIORPC gt 0)*/
	/*and (PRTONE ne .)*/
	;
if nitem eq '12' then nitem='2.02';
if nitem eq '2.02' then EAitem=1; else EAitem=0;

if DATE_DIFF in (0,1) then COV=1; else COV=0;

if EAPC_wsj eq . then EAPC_wsj=0;
if lnEAPC_wsj eq . then lnEAPC_wsj=0;
if EAPC eq . then EAPC=0;
if lnEAPC eq . then lnEAPC=0;
if lnPRIORPC eq . then lnPRIORPC=0;
if INSTOWN eq . then INSTOWN=0;

if SURP lt 0 then BADNEWS=1; else BADNEWS=0;
if EPSPXQ lt 0 then LOSS=1; else LOSS=0;
NONLINEAR=SURP*abs(SURP);
ABSretEA=abs(retEA);

if 1 <= month(prdate) <= 3 then cqtr=1;
else if 4 <= month(prdate) <= 6 then cqtr=2;
else if 7 <= month(prdate) <= 9 then cqtr=3;
else if 10 <= month(prdate) <= 12 then cqtr=4;
cyq=pryear+cqtr/10;

ABvolume=ABvolume*100;

run;

*Principal component of press release quality variables;
proc sort data=base2 out=basefactor nodupkey; by myfname EAitem; run;
proc sort data=basefactor; by EAitem pryear; run;
proc factor data=basefactor out=basefactor1 corr nfactors=1 method = principal noprint; var READABLE HARDINFO SPECIFIC; by EAitem pryear; run;

proc sql; create table base3 as select
	a.*, b.Factor1 as TCOMP
	from base2 as a left join basefactor1 as b
	on a.myfname eq b.myfname and a.EAitem eq b.EAitem
	order by a.gvkey, a.myfname, a.EAitem;
quit;

proc sort data=base3; by myfname nitem descending DISS; run;
proc sort data=base3 out=base6nodup nodupkey; by myfname nitem; run;
proc sort data=base3nodup; by gvkey prdate; run;

*Winsorize;
%WT(data=base3nodup, out=base3ea, byvar=EAitem pryear, vars= DISS TONEDIFF1 TONEDIFF2 PRLENGTH PCLENGTH READABLE HARDINFO SPECIFIC TCOMP
ABvolume ABspread ABmia mia IPT05 retEA retPOST SURP dEPS
pre_volume pre_spread pre_mia ABSretEA
SIZE lnSIZE BTM lnBTM RETVOL TURN EMP lnEMP OWN lnOWN INSTOWN ANALYST lnANALYST EAPC lnEAPC lnEAPC_wsj PRIORPC lnPRIORPC DISC8K lnDISC8K BUSY1 lnBUSY1 BUSY2 lnBUSY2
PRIORmRV ABrvolume ABovolume rpct, type = W, pctl = 1 99, drop= N);

*Prep for Table 3 regressions;
data base6ea; set base3ea; where (2004 <= pryear <= 2015) and (EAitem eq 1); run;





******************************************************************************;
************************************ DiD *******************************;
******************************************************************************;


*Clean data for DiD;
data pcpr2; set pcpr1;
where (2004 <= pryear <= 2014) and (CLOSErdq le 4) and (SPMIM eq '10') and (group in (1,2,5,7,8))
	and (lnSIZE ne .) and (lnBTM ne .) and (RETVOL ne .) and (TURN ne .) and (lnEMP ne .) and (lnOWN ne .)
	and (lnANALYST ne .) and (lnDISC8K gt 0) and (retEA ne .);

if nitem eq '12' then nitem='2.02';
if nitem eq '2.02' then EAitem=1; else EAitem=0;

if DATE_DIFF in (0,1) then COV=1; else COV=0;
if nartsDJ ne . then COVdj=1; else COVdj=0;
if nartsWE ne . then COVwe=1; else COVwe=0;

if nartsIND eq . then nartsIND=0;
if ln_nartsIND eq . then ln_nartsIND=0;

if EAPC eq . then EAPC=0;
if lnEAPC eq . then lnEAPC=0;
if lnPRIORPC eq . then lnPRIORPC=0;

if SURP lt 0 then BADNEWS=1; else BADNEWS=0;
if EPSPXQ lt 0 then LOSS=1; else LOSS=0;
NONLINEAR=SURP*abs(SURP);
ABSretEA=abs(retEA);

if 1 <= month(prdate) <= 3 then cqtr=1;
else if 4 <= month(prdate) <= 6 then cqtr=2;
else if 7 <= month(prdate) <= 9 then cqtr=3;
else if 10 <= month(prdate) <= 12 then cqtr=4;
cyq=pryear+cqtr/10;

ABvolume=ABvolume*100;
run;


*Principal component of press release quality variables;
proc sort data=pcpr2 out=prfactor nodupkey; by myfname EAitem; run;
proc sort data=prfactor; by EAitem pryear; run;
proc factor data=prfactor out=prfactor1 corr nfactors=1 method = principal noprint; var READABLE HARDINFO SPECIFIC; by EAitem pryear; run;

proc sql; create table pcpr3 as select
	a.*, b.Factor1 as TCOMP
	from pcpr2 as a left join prfactor1 as b
	on a.myfname eq b.myfname and a.EAitem eq b.EAitem
	order by a.gvkey, a.myfname, a.EAitem;
quit;


* (1) Keep most similar WSJ article and (2) limit to earnings release;
proc sort data=pcpr3; by myfname nitem descending DISS; run;
proc sort data=pcpr3 out=pcpr6nodup nodupkey; by myfname nitem; run;
proc sort data=pcpr3nodup; by gvkey prdate; run;

data pcpr3ea; set pcpr3nodup; where (pryear ge 2004) and (EAitem eq 1); run;

*Winsorize;
%WT(data=pcpr3ea, out=pcpr6ea, byvar=EAitem pryear, vars= DISS TONEDIFF1 PRLENGTH PCLENGTH READABLE HARDINFO SPECIFIC TCOMP
ABvolume ABspread ABmia mia IPT05 retEA retPOST SURP NONLINEAR dEPS
pre_volume pre_spread pre_mia ABSretEA
SIZE lnSIZE BTM lnBTM RETVOL TURN EMP lnEMP OWN lnOWN INSTOWN ANALYST lnANALYST EAPC lnEAPC PRIORPC lnPRIORPC DISC8K lnDISC8K BUSY1 lnBUSY1 BUSY2 lnBUSY2 nartsIND ln_nartsIND
PRIORmRV ABrvolume ABovolume rpct, type = W, pctl = 1 99, drop= N);




/* Proportion of EAs covered for different sized firms

*S&P 500 - 9057/22553=0.402;
data totalEAs; set pcpr6ea; where SPMIM eq '10'; run; data coverEAs; set pcpr6ea; where SPMIM eq '10' and COV eq 1; run;

*S&P 400 - 1468/16758=0.088;
data totalEAs; set pcpr6ea; where SPMIM eq '91'; run; data coverEAs; set pcpr6ea; where SPMIM eq '91' and COV eq 1; run;

*S&P 600 - 725/23144=0.031;
data totalEAs; set pcpr6ea; where SPMIM eq '92'; run; data coverEAs; set pcpr6ea; where SPMIM eq '92' and COV eq 1; run;
*/

*Figure 1;
*Number of EAs covered by WSJ over time;
data tsCOVwsj; set pcpr6ea; where (COV eq 1) and (2004 <= cyq <= 2015); run;
proc sort data=tsCOVwsj; by cyq; run;
proc freq data=tsCOVwsj; tables cyq / out=FREQwsj; run;

*Number of EAs covered by "WEB" (non-WSJ) outlets over time;
data tsCOVwe; set pcpr6ea; where (COVwe eq 1) and (2007 <= cyq <= 2015); run;
proc sort data=tsCOVwe; by cyq; run;
proc freq data=tsCOVwe; tables cyq / out=FREQwe; run;

***********************;
*Table 3 - Change in coverage regressions;
***********************;

*WSJ;
proc sort data=pcpr6ea; by gvkey prdate; run;
data dreg0; set pcpr6ea; CO=COV; run;
data dreg1; set dreg0; where (2004 <= cyq <= 2015);
lag4gvkey=lag4(gvkey);
lag4CO=lag4(CO); lag3CO=lag3(CO); lag2CO=lag2(CO); lag1CO=lag1(CO);
run;
proc sort data=dreg1; by gvkey descending prdate; run;
data dreg2; set dreg1;
lead1gvkey=lag1(gvkey);
lead1CO=lag1(CO);
run;
proc sort data=dreg2; by gvkey prdate; run;
data dreg3; set dreg2;
if lag4gvkey ne gvkey then lag4CO=.;
if lag4gvkey ne gvkey then lag3CO=.;
if lag4gvkey ne gvkey then lag2CO=.;
if lag4gvkey ne gvkey then lag1CO=.;
if lead1gvkey ne gvkey then lead1CO=.;
drop lag4gvkey lead1gvkey;
run;
data dreg4; set dreg3;
dCO=lag1CO-lag3CO; /* dCO=((lead1CO+CO+lag1CO)/3)-((lag2CO+lag3CO+lag4CO)/3); */
if cyq eq 2007.2 then event1=1; else event1=0;
if cyq eq 2008.2 then event2=1; else event2=0;
if cyq eq 2014.1 then event3=1; else event3=0;
run;
proc surveyreg data=dreg4; cluster gvkey; model dCO = event1 event2 event3; run;quit;

*Other financial outlets;
proc sort data=pcpr6ea; by gvkey prdate; run;
data dreg0; set pcpr6ea; CO=COVwe; run;
data dreg1; set dreg0; where (2007 <= cyq <= 2015);
lag4gvkey=lag4(gvkey);
lag4CO=lag4(CO); lag3CO=lag3(CO); lag2CO=lag2(CO); lag1CO=lag1(CO);
run;
proc sort data=dreg1; by gvkey descending prdate; run;
data dreg2; set dreg1;
lead1gvkey=lag1(gvkey);
lead1CO=lag1(CO);
run;
proc sort data=dreg2; by gvkey prdate; run;
data dreg3; set dreg2;
if lag4gvkey ne gvkey then lag4CO=.;
if lag4gvkey ne gvkey then lag3CO=.;
if lag4gvkey ne gvkey then lag2CO=.;
if lag4gvkey ne gvkey then lag1CO=.;
if lead1gvkey ne gvkey then lead1CO=.;
drop lag4gvkey lead1gvkey;
run;
data dreg4; set dreg3;
dCO=lag1CO-lag3CO; /* dCO=((lead1CO+CO+lag1CO)/3)-((lag2CO+lag3CO+lag4CO)/3); */
if cyq eq 2007.2 then event1=1; else event1=0;
if cyq eq 2008.2 then event2=1; else event2=0;
if cyq eq 2014.1 then event3=1; else event3=0;
run;
proc surveyreg data=dreg4; cluster gvkey; model dCO = event2 event3; run;quit;


*Tables 1 and 2;

*Descriptive statistics for treatment and control groups;
data tEA; set pcpr6ea; where (2006 <= cyq <= 2009) or (2012.4 <= cyq <= 2014.3); run;

data tEApre1; set tEA; where cyq eq 2006.3; run; 
proc sort data=tEApre1; by gvkey descending COV; run; proc sort data=tEApre1 nodupkey; by gvkey; run;
data tEApost1; set tEA; where cyq eq 2007.2; run;
proc sort data=tEApost1; by gvkey descending COV; run; proc sort data=tEApost1 nodupkey; by gvkey; run;
data tEApre2; set tEA; where cyq eq 2007.3; run; 
proc sort data=tEApre2; by gvkey descending COV; run; proc sort data=tEApre2 nodupkey; by gvkey; run;
data tEApost2; set tEA; where cyq eq 2008.2; run;
proc sort data=tEApost2; by gvkey descending COV; run; proc sort data=tEApost2 nodupkey; by gvkey; run;
data tEApre3; set tEA; where cyq eq 2013.2; run; 
proc sort data=tEApre3; by gvkey descending COV; run; proc sort data=tEApre3 nodupkey; by gvkey; run;
data tEApost3; set tEA; where cyq eq 2014.1; run;
proc sort data=tEApost3; by gvkey descending COV; run; proc sort data=tEApost3 nodupkey; by gvkey; run;

proc sql;
create table tEA_1 as select a.*, b.COV as treatPRE1, b.ANALYST as ANALYSTpre1, b.PRIORmRV as PRIORmRVpre1 from tEA as a left join tEApre1 as b on a.gvkey eq b.gvkey;
create table tEA_2 as select a.*, b.COV as treatPOST1, b.ANALYST as ANALYSTpost1, b.PRIORmRV as PRIORmRVpost1 from tEA_1 as a left join tEApost1 as b on a.gvkey eq b.gvkey;
create table tEA_3 as select a.*, b.COV as treatPRE2, b.ANALYST as ANALYSTpre2, b.PRIORmRV as PRIORmRVpre2 from tEA_2 as a left join tEApre2 as b on a.gvkey eq b.gvkey;
create table tEA_4 as select a.*, b.COV as treatPOST2, b.ANALYST as ANALYSTpost2, b.PRIORmRV as PRIORmRVpost2 from tEA_3 as a left join tEApost2 as b on a.gvkey eq b.gvkey;
create table tEA_5 as select a.*, b.COV as treatPRE3, b.ANALYST as ANALYSTpre3, b.PRIORmRV as PRIORmRVpre3 from tEA_4 as a left join tEApre3 as b on a.gvkey eq b.gvkey;
create table tEA_6 as select a.*, b.COV as treatPOST3, b.ANALYST as ANALYSTpost3, b.PRIORmRV as PRIORmRVpost3 from tEA_5 as a left join tEApost3 as b on a.gvkey eq b.gvkey
order by a.gvkey, a.prdate;
quit;

data tEA_7; set tEA_6;

if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq 1 and treatPOST1 eq 0 then trt_ctrAN="TREATMENT";
else if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq 1 and treatPOST1 eq 1 then trt_ctrAN="ALWAYS";
else if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq 0 and treatPOST1 eq 0 then delete; *trt_ctrAN="NEVER";
else if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq . and treatPOST1 eq 0 then delete;
else if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq 0 and treatPOST1 eq . then delete;
else if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq . and treatPOST1 eq . then delete;
else if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq . and treatPOST1 eq 1 then delete;
else if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq 1 and treatPOST1 eq . then delete;
else if 2006.1 <= cyq <= 2007.2 and treatPRE1 eq 0 and treatPOST1 eq 1 then delete;

if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq 0 and treatPOST2 eq 1 then trt_ctrAN="TREATMENT";
else if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq 1 and treatPOST2 eq 1 then delete; *trt_ctrAN="ALWAYS";
else if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq 0 and treatPOST2 eq 0 then trt_ctrAN="NEVER";
else if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq . and treatPOST2 eq 1 then delete;
else if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq 1 and treatPOST2 eq . then delete;
else if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq . and treatPOST2 eq 0 then delete;
else if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq 0 and treatPOST2 eq . then delete;
else if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq . and treatPOST2 eq . then delete;
else if 2007.3 <= cyq <= 2008.4 and treatPRE2 eq 1 and treatPOST2 eq 0 then delete;

if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq 0 and treatPOST3 eq 1 then trt_ctrAN="TREATMENT";
else if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq 1 and treatPOST3 eq 1 then delete; *trt_ctrAN="ALWAYS";
else if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq 0 and treatPOST3 eq 0 then trt_ctrAN="NEVER";
else if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq . and treatPOST3 eq 1 then delete;
else if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq 1 and treatPOST3 eq . then delete;
else if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq . and treatPOST3 eq 0 then delete;
else if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq 0 and treatPOST3 eq . then delete;
else if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq . and treatPOST3 eq . then delete;
else if 2012.4 <= cyq <= 2014.3 and treatPRE3 eq 1 and treatPOST3 eq 0 then delete;

if trt_ctrAN="TREATMENT" then trt_ctr="TREATMENT"; else trt_ctr="CONTROL";
run;
proc freq data=tEA_7; tables trt_ctrAN; run;
proc sort data=tEA_7; by cyq; run; proc freq data=tEA_7; tables trt_ctrAN; by cyq; run;
proc means data=tEA_7 mean std p25 median p75 nolabels; vars retEA SURP IPT05 ABvolume /*EAPC*/ PRIORPC ANALYST SIZE BTM RETVOL /*TURN*/ EMP OWN INSTOWN DISC8K TCOMP BADNEWS /*BUSY1 BUSY2 nartsIND*/; run;

proc univariate data=tEA_7 noprint; var IPT05; output out=data1 pctlpre=P_ pctlpts= 50 to 70 by 1; run; proc print data=data1; run;
proc univariate data=tEA_7 noprint; var ABvolume; output out=data1 pctlpre=P_ pctlpts= 50 to 70 by 1; run; proc print data=data1; run;

*TREATMENT v. ALWAYS v. NEVER;
data tEA_desc; set tEA_7; gvkeyN=gvkey/1; run;
proc sql; create table descGROUPS as select distinct
	gvkeyN as group, trt_ctrAN,
	mean(SIZE) as SIZE, mean(BTM) as BTM, mean(RETVOL) as RETVOL, mean(TURN) as TURN, mean(EMP) as EMP, mean(OWN) as OWN, mean(INSTOWN) as INSTOWN,
	mean(ANALYST) as ANALYST, mean(EAPC) as EAPC, mean(PRIORPC) as PRIORPC, mean(DISC8K) as DISC8K, mean(TCOMP) as TCOMP, mean(BADNEWS) as BADNEWS, mean(BUSY1) as BUSY1
	from tEA_desc group by gvkeyN, trt_ctrAN order by gvkeyN, trt_ctrAN;
	quit;
proc sql; create table descQTRS as select distinct
	cyq as group, trt_ctrAN,
	mean(SIZE) as SIZE, mean(BTM) as BTM, mean(RETVOL) as RETVOL, mean(TURN) as TURN, mean(EMP) as EMP, mean(OWN) as OWN, mean(INSTOWN) as INSTOWN,
	mean(ANALYST) as ANALYST, mean(EAPC) as EAPC, mean(PRIORPC) as PRIORPC, mean(DISC8K) as DISC8K, mean(TCOMP) as TCOMP, mean(BADNEWS) as BADNEWS, mean(BUSY1) as BUSY1
	from tEA_desc group by cyq, trt_ctrAN order by cyq, trt_ctrAN;
	quit;
proc append base=descGROUPS data=descQTRS force; run;

proc sort data=descGROUPS; by trt_ctrAN; run;
proc means data=descGROUPS mean nolabels; by trt_ctrAN; vars SIZE BTM RETVOL TURN EMP OWN INSTOWN ANALYST EAPC PRIORPC DISC8K TCOMP BADNEWS BUSY1; run;

*TREATMENT v. CONTROL;
data tEA_desc; set tEA_7; gvkeyN=gvkey/1; run;
proc sql; create table descGROUPS as select distinct
	gvkeyN as group, trt_ctr,
	mean(SIZE) as SIZE, mean(BTM) as BTM, mean(RETVOL) as RETVOL, mean(TURN) as TURN, mean(EMP) as EMP, mean(OWN) as OWN, mean(INSTOWN) as INSTOWN,
	mean(ANALYST) as ANALYST, mean(EAPC) as EAPC, mean(PRIORPC) as PRIORPC, mean(DISC8K) as DISC8K, mean(TCOMP) as TCOMP, mean(BADNEWS) as BADNEWS, mean(BUSY1) as BUSY1
	from tEA_desc group by gvkeyN, trt_ctr order by gvkeyN, trt_ctr;
	quit;
proc sql; create table descQTRS as select distinct
	cyq as group, trt_ctr,
	mean(SIZE) as SIZE, mean(BTM) as BTM, mean(RETVOL) as RETVOL, mean(TURN) as TURN, mean(EMP) as EMP, mean(OWN) as OWN, mean(INSTOWN) as INSTOWN,
	mean(ANALYST) as ANALYST, mean(EAPC) as EAPC, mean(PRIORPC) as PRIORPC, mean(DISC8K) as DISC8K, mean(TCOMP) as TCOMP, mean(BADNEWS) as BADNEWS, mean(BUSY1) as BUSY1
	from tEA_desc group by cyq, trt_ctr order by cyq, trt_ctr;
	quit;
proc append base=descGROUPS data=descQTRS force; run;

proc sort data=descGROUPS; by trt_ctr; run;
proc means data=descGROUPS mean nolabels; by trt_ctr; vars SIZE BTM RETVOL TURN EMP OWN INSTOWN ANALYST EAPC PRIORPC DISC8K TCOMP BADNEWS BUSY1; run;
*Comparison of group means;
proc ttest data=descGROUPS plots=none; class trt_ctr; var SIZE; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var BTM; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var RETVOL; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var TURN; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var EMP; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var OWN; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var INSTOWN; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var ANALYST; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var EAPC; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var PRIORPC; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var DISC8K; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var TCOMP; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var BADNEWS; ods select Statistics TTests; run;
proc ttest data=descGROUPS plots=none; class trt_ctr; var BUSY1; ods select Statistics TTests; run;


***************;
* Main results ;
***************;

*Aggregate experiments (treatment based on difference between coverage in q-2 and q+2);
proc sort data=tEA_7; by cyq; run;

*Create time, industry, and pseudo-firm dummies;
%dummy( 
   data=tEA_7,    /* name of input dataset                  */
   out=tEA_7_t,       /* name of output dataset                 */
   var= cyq,           /* variable(s) to be dummied              */
   base=_FIRST_,     /* base category                          */
   prefix = D_,     /* prefix for dummy variable names        */
   format =,        /* format used to categorize variable     */
   name  = ,     /* VAL: variable names are D_value        */
   fullrank=1       /* Eliminate dummy for baseline category? */
   );
%dummy( 
   data=tEA_7_t,    /* name of input dataset                  */
   out=tEA_7_ti,       /* name of output dataset                 */
   var= industry12,           /* variable(s) to be dummied              */
   base=_FIRST_,     /* base category                          */
   prefix = I_,     /* prefix for dummy variable names        */
   format =,        /* format used to categorize variable     */
   name  = ,     /* VAL: variable names are D_value        */
   fullrank=1       /* Eliminate dummy for baseline category? */
   );
%dummy( 
   data=tEA_7_ti,    /* name of input dataset                  */
   out=tEA_7_tip,       /* name of output dataset                 */
   var= pseudo,           /* variable(s) to be dummied              */
   base=_FIRST_,     /* base category                          */
   prefix = P_,     /* prefix for dummy variable names        */
   format =,        /* format used to categorize variable     */
   name  = ,     /* VAL: variable names are D_value        */
   fullrank=1       /* Eliminate dummy for baseline category? */
   );


*Get DISS residuals;
data tEA_8_wDISSresids1; set tEA_7_tip; lPRLENGTH=log(PRLENGTH); lPCLENGTH=log(PCLENGTH); lPRLENGTH2=lPRLENGTH*lPRLENGTH; lPCLENGTH2=lPCLENGTH*lPCLENGTH; run;
proc reg data=tEA_8_wDISSresids1; model DISS = lPRLENGTH lPCLENGTH /*lPRLENGTH2 lPCLENGTH2*/ D_1-D_19 ; output out=tEA_8_wDISSresids2 r=residDISS; run; quit;

proc rank data=tEA_8_wDISSresids2 out=tEA_8_wDISSresids3 groups=5; var residDISS TONEDIFF1; ranks rresidDISS rTONEDIFF1; run;

*Create treatment and "post" indicators, interactions, and dummies;
data tEA_9; set tEA_8_wDISSresids3; *where abs(CAR05) ge 0.02; /* uncomment preceding where statement for IPT analyses */
if 2006.1 <= cyq <= 2006.4 then cov=1; else if 2008.1 <= cyq <= 2008.4 then cov=1; else if 2013.4 <= cyq <= 2014.3 then cov=1; else cov=0;
if trt_ctr="TREATMENT" then treat=1; else treat=0;
treatXcov=treat*cov;

if trt_ctr="TREATMENT" and cyq eq 2007.1 then pre1=1;else if trt_ctr="TREATMENT" and cyq eq 2007.4 then pre1=1; else if trt_ctr="TREATMENT" and cyq eq 2013.3 then pre1=1; else pre1=0;
if trt_ctr="TREATMENT" and cyq eq 2007.2 then pre2=1; else if trt_ctr="TREATMENT" and cyq eq 2007.3 then pre2=1; else if trt_ctr="TREATMENT" and cyq eq 2013.2 then pre2=1; else pre2=0;
if trt_ctr="TREATMENT" and cyq eq 2013.1 then pre3=1; else pre3=0; /* only two quarters of pre period for first and second events */
if trt_ctr="TREATMENT" and cyq eq 2012.4 then pre4=1; else pre4=0;

if trt_ctr="TREATMENT" and cyq eq 2006.1 then post1=1;else if trt_ctr="TREATMENT" and cyq eq 2008.1 then post1=1; else if trt_ctr="TREATMENT" and cyq eq 2013.4 then post1=1; else post1=0;
if trt_ctr="TREATMENT" and cyq eq 2006.2 then post2=1; else if trt_ctr="TREATMENT" and cyq eq 2008.2 then post2=1; else if trt_ctr="TREATMENT" and cyq eq 2014.1 then post2=1; else post2=0;
if trt_ctr="TREATMENT" and cyq eq 2006.3 then post3=1; else if trt_ctr="TREATMENT" and cyq eq 2008.3 then post3=1; else if trt_ctr="TREATMENT" and cyq eq 2014.2 then post3=1; else post3=0;
if trt_ctr="TREATMENT" and cyq eq 2006.4 then post4=1; else if trt_ctr="TREATMENT" and cyq eq 2008.4 then post4=1; else if trt_ctr="TREATMENT" and cyq eq 2014.3 then post4=1; else post4=0;

SURPXtreatXcov=SURP*treat*cov; SURPXtreat=SURP*treat; SURPXcov=SURP*cov;

SURPXtreatXpre4=SURP*treat*pre4; SURPXtreatXpre3=SURP*treat*pre3; SURPXtreatXpre2=SURP*treat*pre2; SURPXtreatXpre1=SURP*treat*pre1;
SURPXtreatXpost4=SURP*treat*post4; SURPXtreatXpost3=SURP*treat*post3; SURPXtreatXpost2=SURP*treat*post2; SURPXtreatXpost1=SURP*treat*post1;
treatXpre4=treat*pre4; treatXpre3=treat*pre3; treatXpre2=treat*pre2; treatXpre1=treat*pre1;
treatXpost4=treat*post4; treatXpost3=treat*post3; treatXpost2=treat*post2; treatXpost1=treat*post1;

SURPXTCOMP=SURP*TCOMP; SURPXBADNEWS=SURP*BADNEWS; SURPXPRLENGTH=SURP*PCLENGTH; SURPXPCLENGTH=SURP*PRLENGTH; SURPXlnSIZE=SURP*lnSIZE; SURPXlnBTM=SURP*lnBTM; SURPXRETVOL=SURP*RETVOL;
SURPXTURN=SURP*TURN; SURPXlnEMP=SURP*lnEMP; SURPXlnOWN=SURP*lnOWN; SURPXINSTOWN=SURP*INSTOWN; SURPXlnANALYST=SURP*lnANALYST; SURPXlnEAPC=SURP*lnEAPC; SURPXlnPRIORPC=SURP*lnPRIORPC; SURPXlnDISC8K=SURP*lnDISC8K;

/* comment following line for Tone Diff analyses */
ANLYS=((rresidDISS)/(-5))+1; if ANLYS eq . then ANLYS=0;

/* uncomment following line for Tone Diff analyses */
/* if TONEDIFF1 ne . then ANLYS=TONEDIFF1*(-100); else ANLYS=0; */

if rresidDISS eq 4 then ANLYS1=1; else ANLYS1=0; if rresidDISS eq 3 then ANLYS2=1; else ANLYS2=0; if rresidDISS eq 2 then ANLYS3=1; else ANLYS3=0;
if rresidDISS eq 1 then ANLYS4=1; else ANLYS4=0; if rresidDISS eq 0 then ANLYS5=1; else ANLYS5=0;

treatXcovXANLYS=treat*cov*ANLYS; treatXANLYS=treat*ANLYS; covXANLYS=cov*ANLYS; 
SURPXtreatXcovXANLYS=SURP*treat*cov*ANLYS; SURPXtreatXANLYS=SURP*treat*ANLYS; SURPXcovXANLYS=SURP*cov*ANLYS; SURPXANLYS=SURP*ANLYS;

treatXcovXLNBUSY2=treat*cov*LNBUSY2; treatXLNBUSY2=treat*LNBUSY2; covXLNBUSY2=cov*LNBUSY2; 
SURPXtreatXcovXLNBUSY2=SURP*treat*cov*LNBUSY2; SURPXtreatXLNBUSY2=SURP*treat*LNBUSY2; SURPXcovXLNBUSY2=SURP*cov*LNBUSY2; SURPXLNBUSY2=SURP*LNBUSY2;

treatXcovXLN_NARTSIND=treat*cov*LN_NARTSIND; treatXLN_NARTSIND=treat*LN_NARTSIND; covXLN_NARTSIND=cov*LN_NARTSIND; 
SURPXtreatXcovXLN_NARTSIND=SURP*treat*cov*LN_NARTSIND; SURPXtreatXLN_NARTSIND=SURP*treat*LN_NARTSIND; SURPXcovXLN_NARTSIND=SURP*cov*LN_NARTSIND; SURPXLN_NARTSIND=SURP*LN_NARTSIND;

if PRLENGTH eq . then PRLENGTH=0; if PCLENGTH eq . then PCLENGTH=0;

*Create interactions between surprise and controls and fixed effects;
SURPXtreatXcovXBADNEWS=SURP*treat*cov*BADNEWS; treatXcovXBADNEWS=treat*cov*BADNEWS; SURPXtreatXBADNEWS=SURP*treat*BADNEWS; SURPXcovXBADNEWS=SURP*cov*BADNEWS; treatXBADNEWS=treat*BADNEWS;
SURPXtreatXcovXLOSS=SURP*treat*cov*LOSS; treatXcovXLOSS=treat*cov*LOSS; SURPXtreatXLOSS=SURP*treat*LOSS; SURPXcovXLOSS=SURP*cov*LOSS; treatXLOSS=treat*LOSS; SURPxLOSS=SURP*LOSS;
treatXcovXNONLINEAR=treat*cov*NONLINEAR; SURPXtreatXNONLINEAR=SURP*treat*NONLINEAR; SURPXcovXNONLINEAR=SURP*cov*NONLINEAR; treatXNONLINEAR=treat*NONLINEAR;

array D_ {19} D_1-D_19;
array SURPxD_    {19} SURPxD_1 - SURPxD_19;
do _k_=1 to 19; SURPxD_{_k_} = SURP * D_{_k_}; end;
drop _k_;

array I_ {11} I_1-I_11;
array SURPxI_    {11} SURPxI_1 - SURPxI_11;
do _k_=1 to 11; SURPxI_{_k_} = SURP * I_{_k_}; end;
drop _k_;

array P_ {68} P_1-P_68;
array SURPxP_    {68} SURPxP_1 - SURPxP_68;
do _k_=1 to 68; SURPxP_{_k_} = SURP * P_{_k_}; end;
drop _k_;

id = _n_;
run;


*Implement coarsened exact matching: 1) SURP PRIORmRV 2) PRIORPC ANALYST 3) EAPC BUSY1;
proc standard data=tEA_9 out=idmatches mean=0 std=1; var SURP PRIORmRV PRIORPC ANALYST EAPC BUSY1; run;
data coarse1; set idmatches; run;
%CEM(
			lib = work,
			dataset = coarse1,
			ID = id ,
			treat = treat,
			keep = SURP PRIORmRV,
			DEL_miss = 0,
			MATCH_TYPE = N,
			method = Sturges,
			path_graph = path_graph,
			report = on
			);
data tEA_9_coarse; set coarse1; where matched ne .; run;

/* Create dataset with main identifiers

proc sort data=tEA_9_coarse out=identifiers nodupkey; by gvkey; run;
data identifiers1; set identifiers; keep gvkey cusip permno; run;
proc export data=identifiers1 dbms=xlsx outfile="C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Analysis\identifiers.xlsx" replace; run;

*/


*Standardize variables for ERC regressions;
proc standard data=tEA_9_coarse out=tEA_9_std mean=0 std=1;
var ABvolume IPT05 retEA retPOST ABspread ABmia mia
treatXcov treat cov TCOMP BADNEWS PRLENGTH PCLENGTH lnSIZE lnBTM RETVOL TURN lnEMP lnOWN INSTOWN lnANALYST lnEAPC lnPRIORPC lnDISC8K
SURP SURPXTCOMP SURPXBADNEWS SURPXPRLENGTH SURPXPCLENGTH SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXTURN SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnANALYST SURPXlnEAPC SURPXlnPRIORPC SURPXlnDISC8K
SURPXtreatXcov SURPXtreat SURPXcov

SURPXtreatXpre4 SURPXtreatXpre3 SURPXtreatXpre2 SURPXtreatXpre1
SURPXtreatXpost4 SURPXtreatXpost3 SURPXtreatXpost2 SURPXtreatXpost1
treatXpre4 treatXpre3 treatXpre2 treatXpre1
treatXpost4 treatXpost3 treatXpost2 treatXpost1

SURPXtreatXcovXBADNEWS treatXcovXBADNEWS  SURPXtreatXBADNEWS SURPXcovXBADNEWS treatXBADNEWS
SURPXtreatXcovXLOSS treatXcovXLOSS SURPXtreatXLOSS SURPXcovXLOSS treatXLOSS SURPxLOSS
treatXcovXNONLINEAR SURPXtreatXNONLINEAR SURPXcovXNONLINEAR treatXNONLINEAR

treatXcovXANLYS treatXANLYS covXANLYS SURPXtreatXcovXANLYS SURPXtreatXANLYS SURPXcovXANLYS SURPXANLYS
treatXcovXLNBUSY2 treatXLNBUSY2 covXLNBUSY2 SURPXtreatXcovXLNBUSY2 SURPXtreatXLNBUSY2 SURPXcovXLNBUSY2 SURPXLNBUSY2
treatXcovXLN_NARTSIND treatXLN_NARTSIND covXLN_NARTSIND SURPXtreatXcovXLN_NARTSIND SURPXtreatXLN_NARTSIND SURPXcovXLN_NARTSIND SURPXLN_NARTSIND

PRLENGTH PCLENGTH

D_1-D_19 SURPxD_1-SURPxD_19
I_1-I_11 SURPxI_1-SURPxI_11
P_1-P_68 SURPxP_1-SURPxP_68
;
run;

*Standardize variables AND use within regression to implement firm fixed effects for IPT and abnormal volume regressions;
proc sort data=tEA_9_coarse; by gvkey; run;
proc standard data=tEA_9_coarse out=tEA_9_ffe mean=0 std=1; by gvkey;
var ABvolume IPT05 retEA retPOST ABspread ABmia mia
treatXcov treat cov TCOMP BADNEWS PRLENGTH PCLENGTH lnSIZE lnBTM RETVOL TURN lnEMP lnOWN INSTOWN lnANALYST lnEAPC lnPRIORPC lnDISC8K
SURP SURPXTCOMP SURPXBADNEWS SURPXPRLENGTH SURPXPCLENGTH SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXTURN SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnANALYST SURPXlnEAPC SURPXlnPRIORPC SURPXlnDISC8K
SURPXtreatXcov SURPXtreat SURPXcov

SURPXtreatXpre4 SURPXtreatXpre3 SURPXtreatXpre2 SURPXtreatXpre1
SURPXtreatXpost4 SURPXtreatXpost3 SURPXtreatXpost2 SURPXtreatXpost1
treatXpre4 treatXpre3 treatXpre2 treatXpre1
treatXpost4 treatXpost3 treatXpost2 treatXpost1

SURPXtreatXcovXBADNEWS treatXcovXBADNEWS  SURPXtreatXBADNEWS SURPXcovXBADNEWS treatXBADNEWS
SURPXtreatXcovXLOSS treatXcovXLOSS SURPXtreatXLOSS SURPXcovXLOSS treatXLOSS SURPxLOSS
treatXcovXNONLINEAR SURPXtreatXNONLINEAR SURPXcovXNONLINEAR treatXNONLINEAR

treatXcovXANLYS treatXANLYS covXANLYS SURPXtreatXcovXANLYS SURPXtreatXANLYS SURPXcovXANLYS SURPXANLYS
treatXcovXLNBUSY2 treatXLNBUSY2 covXLNBUSY2 SURPXtreatXcovXLNBUSY2 SURPXtreatXLNBUSY2 SURPXcovXLNBUSY2 SURPXLNBUSY2
treatXcovXLN_NARTSIND treatXLN_NARTSIND covXLN_NARTSIND SURPXtreatXcovXLN_NARTSIND SURPXtreatXLN_NARTSIND SURPXcovXLN_NARTSIND SURPXLN_NARTSIND

PRLENGTH PCLENGTH

D_1-D_19 SURPxD_1-SURPxD_19
I_1-I_11 SURPxI_1-SURPxI_11
P_1-P_68 SURPxP_1-SURPxP_68
;
run;


*Table 4 ERC;

proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP D_1-D_19;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat D_1-D_19;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS 
SURPXlnPRIORPC SURPXlnANALYST SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnDISC8K SURPXTCOMP SURPXBADNEWS D_1-D_19;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS 
SURPXlnPRIORPC SURPXlnANALYST SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnDISC8K SURPXTCOMP SURPXBADNEWS D_1-D_19 SURPxD_1-SURPxD_19;run;quit;



/* industry or pseudo-firm fixed effects;

*industry;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP D_1-D_19 I_1-I_11;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat D_1-D_19 I_1-I_11;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS 
SURPXlnPRIORPC SURPXlnANALYST SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnDISC8K SURPXTCOMP SURPXBADNEWS D_1-D_19 I_1-I_11;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS 
SURPXlnPRIORPC SURPXlnANALYST SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnDISC8K SURPXTCOMP SURPXBADNEWS D_1-D_19 SURPxD_1-SURPxD_19 I_1-I_11 SURPxI_1-SURPxI_11;run;quit;

*pseudo-firm;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP D_1-D_19 P_1-P_68;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat D_1-D_19 P_1-P_68;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS 
SURPXlnPRIORPC SURPXlnANALYST SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnDISC8K SURPXTCOMP SURPXBADNEWS D_1-D_19 P_1-P_68;run;quit;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP SURPXtreatXcov SURPXtreat SURPXcov treatXcov treat lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS 
SURPXlnPRIORPC SURPXlnANALYST SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnDISC8K SURPXTCOMP SURPXBADNEWS D_1-D_19 SURPxD_1-SURPxD_19 P_1-P_68 SURPxP_1-SURPxP_68;run;quit;

*/

*Table 7;
*Price discovery - IPT; *Require abs(CAR05) ge 0.02 above;
proc surveyreg data=tEA_9_ffe; cluster gvkey; model IPT05 = treatXcov D_1-D_19;run;quit;
proc surveyreg data=tEA_9_ffe; cluster gvkey; model IPT05 = treatXcov lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnANALYST lnEAPC lnPRIORPC lnDISC8K TCOMP BADNEWS D_1-D_19;run;quit;

*Abnormal Volume;
proc surveyreg data=tEA_9_ffe; cluster gvkey; model ABvolume = treatXcov D_1-D_19;run;quit;
proc surveyreg data=tEA_9_ffe; cluster gvkey; model ABvolume = treatXcov lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnANALYST lnEAPC lnPRIORPC lnDISC8K TCOMP BADNEWS D_1-D_19;run;quit;


**************************;
*Creation v. Dissemination;
**************************;

*Instrumental Variables;
/* lnBUSY1 = number of competing earnings announcements, lnBUSY2 = value-weighted earnings announcements, ln_nartsIND = media coverage of industry (WEB edition) */

*Table 5;
*Descriptives;
data nomiss_ANLYS; set tEA_9; where ANLYS ne 0; run;
proc means data=nomiss_ANLYS n mean std q1 median q3; var ANLYS BUSY1 BUSY2 lnBUSY2 nartsIND ln_nartsIND; run;
proc corr data=nomiss_ANLYS pearson spearman; var lnBUSY2 ln_nartsIND; run;

proc surveyreg data=tEA_9; cluster gvkey; model ANLYS = lnBUSY2 D_1-D_19;run;quit;
proc surveyreg data=tEA_9; cluster gvkey; model ANLYS = ln_nartsIND D_1-D_19;run;quit;
proc surveyreg data=tEA_9; cluster gvkey; model ANLYS = lnBUSY2 ln_nartsIND D_1-D_19; run;quit;

proc surveyreg data=tEA_9; cluster gvkey;model ANLYS = lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS D_1-D_19 / ANOVA;run;quit;
proc surveyreg data=tEA_9; cluster gvkey;model ANLYS = lnBUSY2 ln_nartsIND lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS D_1-D_19 / ANOVA;run;quit;

* Word Diff.: F-stat. = ((437.57-416.35)/2)/(416.35/493)=12.56 ;
* Tone Diff.: F-stat. = ((2044.06-1946.49)/2)/(1946.49/493)=12.36 ;

*Table 6;
*ERC;
proc syslin data=tEA_9_std 2SLS;
endogenous treatXcovXANLYS treatXANLYS covXANLYS SURPXtreatXcovXANLYS SURPXtreatXANLYS SURPXcovXANLYS SURPXANLYS;
instruments LNBUSY2 LN_NARTSIND treatXcovXLNBUSY2 treatXLNBUSY2 covXLNBUSY2 SURPXtreatXcovXLNBUSY2 SURPXtreatXLNBUSY2 SURPXcovXLNBUSY2 SURPXLNBUSY2 treatXcovXLN_NARTSIND treatXLN_NARTSIND covXLN_NARTSIND SURPXtreatXcovXLN_NARTSIND SURPXtreatXLN_NARTSIND SURPXcovXLN_NARTSIND SURPXLN_NARTSIND;
model retEA = SURP SURPXtreatXcov SURPXtreatXcovXANLYS SURPXtreatXANLYS SURPXcovXANLYS treatXcovXANLYS SURPXtreat SURPXcov SURPXANLYS treatXcov treatXANLYS covXANLYS treat lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnANALYST lnEAPC lnPRIORPC lnDISC8K TCOMP BADNEWS SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnANALYST SURPXlnEAPC SURPXlnPRIORPC SURPXlnDISC8K SURPXTCOMP SURPXBADNEWS D_1-D_19 SURPxD_1-SURPxD_19 / overid;
run;quit;

*Table 7;
*IPT; *Require abs(CAR05) ge 0.02 above;
proc syslin data=tEA_9_ffe 2SLS;
endogenous treatXcovXANLYS treatXANLYS covXANLYS;
instruments LNBUSY2 LN_NARTSIND treatXcovXLNBUSY2 treatXLNBUSY2 covXLNBUSY2 treatXcovXLN_NARTSIND treatXLN_NARTSIND covXLN_NARTSIND;
model IPT05 = treatXcov treatXcovXANLYS treatXANLYS covXANLYS lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnANALYST lnEAPC lnPRIORPC lnDISC8K TCOMP BADNEWS D_1-D_19 / overid;
run;quit;
*Abnormal Volume;
proc syslin data=tEA_9_ffe 2SLS;
endogenous treatXcovXANLYS treatXANLYS covXANLYS;
instruments LNBUSY2 LN_NARTSIND treatXcovXLNBUSY2 treatXLNBUSY2 covXLNBUSY2 treatXcovXLN_NARTSIND treatXLN_NARTSIND covXLN_NARTSIND;
model ABvolume = treatXcov treatXcovXANLYS treatXANLYS covXANLYS lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnANALYST lnEAPC lnPRIORPC lnDISC8K TCOMP BADNEWS D_1-D_19 / overid;
run;quit;


*************************;
*Linguistic Analysis of High and Low Difference Articles;
*************************;

/*
proc import out=mydir.PressCoverage2020 datafile="C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Analysis\PressCoverage2020.xlsx" dbms=excel replace; getnames=yes; run;
*/

data PressCoverage2020; set mydir.PressCoverage2020; drop company_name date firm_article; run;
proc sql; create table linga1 as select
	a.*, b.*
	from tEA_9_coarse as a left join PressCoverage2020 as b
	on a.pc_fname eq b.fname;
quit;
data lingaHIGHLOW; set linga1; where rresidDISS = 4 or rresidDISS = 0; run;

*Table 8;
proc sort data=lingaHIGHLOW; by rresidDISS; run;
proc means data=lingaHIGHLOW mean nolabels; by rresidDISS; vars length readability specificity hsmix negativity similarity context total_q public_q insider_q analyst_q ooutsider_q; run;
*Comparison of group means;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var length; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var readability; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var specificity; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var hsmix; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var negativity; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var similarity; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var context; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var total_q; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var public_q; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var insider_q; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var analyst_q; ods select Statistics TTests; run;
proc ttest data=lingaHIGHLOW plots=none; class rresidDISS; var ooutsider_q; ods select Statistics TTests; run;






*Table 3 - Panels B and C;

*Get change in coverage and absolute changes in control variables;
proc sql; create table pred1 as select
	a.*, (a.COV-b.COV) as dCOV,
	abs(a.SURP-b.SURP) as dSURP, abs(a.lnPRIORPC-b.lnPRIORPC) as dlnPRIORPC, abs(a.lnSIZE-b.lnSIZE) as dlnSIZE, abs(a.lnBTM-b.lnBTM) as dlnBTM, abs(a.RETVOL-b.RETVOL) as dRETVOL, abs(a.lnEMP-b.lnEMP) as dlnEMP,
	abs(a.lnOWN-b.lnOWN) as dlnOWN, abs(a.INSTOWN-b.INSTOWN) as dINSTOWN, abs(a.lnANALYST-b.lnANALYST) as dlnANALYST, abs(a.lnDISC8K-b.lnDISC8K) as dlnDISC8K,
	abs(a.TCOMP-b.TCOMP) as dTCOMP, abs(a.BADNEWS-b.BADNEWS) as dBADNEWS	
	from base6ea as a left join base6ea as b
	on a.gvkey eq b.gvkey
	and intnx('quarter',b.prdate,0) = intnx('quarter',a.prdate,-3)
	order by a.gvkey, a.cyq;
quit;
proc sort data=pred1 nodupkey; by myfname; run;

*Get next quarter surprise;
proc sql; create table pred2 as select
	a.*, b.SURP as fSURP, abs(b.SURP) as afSURP
	from pred1 as a left join base6ea as b
	on a.gvkey eq b.gvkey
	and intnx('quarter',b.prdate,0) = intnx('quarter',a.prdate,1)
	order by a.gvkey, a.cyq;
quit;
proc sort data=pred2 nodupkey; by myfname; run;

*Identify sample changes and endogenous changes;
data pred_s; set pred2; where cyq eq 2007.2 or cyq eq 2008.2 or cyq eq 2014.1; run;
data pred_e; set pred2; where cyq ne 2007.2 and cyq ne 2008.2 and cyq ne 2014.1; run;

data pred_s1; set pred_s; where dCOV ne . and fSURP ne .;
if dCOV ne 0 then changeS=1; else changeS=0;
if dCOV eq -1 then termS=1; else termS=0;
if dCOV eq 1 then origS=1; else origS=0;
changeE=0; termE=0; origE=0;
run;
data pred3; set pred_e; where dCOV ne . and fSURP ne .;
changeS=0; termS=0; origS=0;
if dCOV ne 0 then changeE=1; else changeE=0;
if dCOV eq -1 then termE=1; else termE=0;
if dCOV eq 1 then origE=1; else origE=0;
run;

/*
*For sample and endogenous changes regression (Panels B and C);

data predGO; set pred_s1; run;
or
data predGO; set pred3; run;
*/


proc sort data=predGO; by cyq; run;
%dummy( 
   data=predGO,    /* name of input dataset                  */
   out=pred_8,       /* name of output dataset                 */
   var= cyq,           /* variable(s) to be dummied              */
   base=_FIRST_,     /* base category                          */
   prefix = D_,     /* prefix for dummy variable names        */
   format =,        /* format used to categorize variable     */
   name  = ,     /* VAL: variable names are D_value        */
   fullrank=1       /* Eliminate dummy for baseline category? */
   );


*Panel B - Sample coverage changes; /* only includes the three treatment assignment quarters following the three WSJ events, hence year-quarter dummies are D_1-D_2 */
proc surveyreg data=pred_8; cluster gvkey; model changeS = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_2;run;quit;
proc reg data=pred_8; model changeS = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_2;
test dSURP,dlnPRIORPC,dlnSIZE,dlnBTM,dRETVOL,dlnEMP,dlnOWN,dINSTOWN,dlnANALYST,dlnDISC8K,dTCOMP,dBADNEWS; run;quit;

proc surveyreg data=pred_8; cluster gvkey; model termS = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_2;run;quit;
proc reg data=pred_8; model terms = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_2;
test dSURP,dlnPRIORPC,dlnSIZE,dlnBTM,dRETVOL,dlnEMP,dlnOWN,dINSTOWN,dlnANALYST,dlnDISC8K,dTCOMP,dBADNEWS; run;quit;
proc surveyreg data=pred_8; cluster gvkey; model origS = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_2;run;quit;
proc reg data=pred_8; model origs = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_2;
test dSURP,dlnPRIORPC,dlnSIZE,dlnBTM,dRETVOL,dlnEMP,dlnOWN,dINSTOWN,dlnANALYST,dlnDISC8K,dTCOMP,dBADNEWS; run;quit;


*Panel C - Endogenous coverage changes; /* includes all other quarters (i.e., minus the three "treatment" quarters) during 11 year period from 2004 through 2014, hence year-quarter dummies are D_1-D_40 */
proc surveyreg data=pred_8; cluster gvkey; model changeE = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_40;run;quit;
proc reg data=pred_8; model changeE = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_40;
test dSURP,dlnPRIORPC,dlnSIZE,dlnBTM,dRETVOL,dlnEMP,dlnOWN,dINSTOWN,dlnANALYST,dlnDISC8K,dTCOMP,dBADNEWS; run;quit;

proc surveyreg data=pred_8; cluster gvkey; model termE = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_40;run;quit;
proc reg data=pred_8; model termE = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_40;
test dSURP,dlnPRIORPC,dlnSIZE,dlnBTM,dRETVOL,dlnEMP,dlnOWN,dINSTOWN,dlnANALYST,dlnDISC8K,dTCOMP,dBADNEWS; run;quit;
proc surveyreg data=pred_8; cluster gvkey; model origE = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_40;run;quit;
proc reg data=pred_8; model origE = dSURP dlnPRIORPC dlnANALYST dlnSIZE dlnBTM dRETVOL dlnEMP dlnOWN dINSTOWN dlnDISC8K dTCOMP dBADNEWS D_1-D_40;
test dSURP,dlnPRIORPC,dlnSIZE,dlnBTM,dRETVOL,dlnEMP,dlnOWN,dINSTOWN,dlnANALYST,dlnDISC8K,dTCOMP,dBADNEWS; run;quit;







*******;
*Figure 2: Map out estimations in event time;
*******;

*Attributes table for formatting;
data Attrs2;
length Value $20 LineColor $20 MarkerColor $20 LinePattern $20;
ID = "Jobs2"; 
Value = 'Transparent'; LineColor = "BIGB"; MarkerColor = "BIGB"; LinePattern="MediumDash"; output; /* Cardinal is CXBD2031 */
Value = 'Opaque'; LineColor = "BL"; MarkerColor = "BL"; LinePattern="Solid"; output;
run;

* Regressions with event time indicators;

*Price discovery - ERC;
proc surveyreg data=tEA_9_std; cluster gvkey; model retEA = SURP
SURPXtreatXpre4 SURPXtreatXpre3 SURPXtreatXpre2 SURPXtreatXpost1 SURPXtreatXpost2 SURPXtreatXpost3 SURPXtreatXpost4   
SURPXtreat SURPXcov treatXcov treat lnPRIORPC lnANALYST lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnDISC8K TCOMP BADNEWS 
SURPXlnPRIORPC SURPXlnANALYST SURPXlnSIZE SURPXlnBTM SURPXRETVOL SURPXlnEMP SURPXlnOWN SURPXINSTOWN SURPXlnDISC8K SURPXTCOMP SURPXBADNEWS D_1-D_19 SURPxD_1-SURPxD_19;
ods output parameterestimates=parmERC;
run;quit;

*Price discovery - IPT; *Require abs(CAR05) ge 0.02 above;
proc surveyreg data=tEA_9_ffe; cluster gvkey; model IPT05 = treatXpre4 treatXpre3 treatXpre2 treatXpost1 treatXpost2 treatXpost3 treatXpost4 
lnSIZE lnBTM RETVOL lnEMP lnOWN INSTOWN lnANALYST lnEAPC lnPRIORPC lnDISC8K TCOMP BADNEWS D_1-D_19;
ods output parameterestimates=parmIPT;
run;quit;

*Abnormal Volume;
proc surveyreg data=tEA_9_ffe; where cyq lt 2014.1; cluster gvkey; model ABvolume = treatXpre4 treatXpre1 treatXpost1 treatXpost2 treatXpost3 treatXpost4   
 D_1-D_19;
ods output parameterestimates=parmVOL;
run;quit;


*******;
* Fig 2A - ERC;
*******;

data figERC; length ETI $ 3; set parmERC;
where Parameter in ("SURPXtreatXpre4","SURPXtreatXpre3","SURPXtreatXpre2","SURPXtreatXpost1","SURPXtreatXpost2","SURPXtreatXpost3","SURPXtreatXpost4");
q=_n_ - 5; if q > -2 then q+1;
LowerCLMean = Estimate-(StdErr*2); UpperCLMean = Estimate+(StdErr*2);
if q < 0 then ETI= cats('q',put(q,2.)); else if q > 0 then ETI= cats('q+',put(q,2.)); else ETI='q';
drop DenDF tValue Probt;
run;

data figERC1;length Parameter $ 16;set figERC end=eof;output;
if eof then do; Parameter='SURPXtreatXpre1'; q=-1; Estimate=0;StdErr=0;LowerCLMean=0;UpperCLMean=0;ETI='q-1';
output;end;run;
proc sort data=figERC1; by q; run;

ods listing gpath="C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Analysis";
ods graphics on / reset=index imagename="Fig2ERC" imagefmt=jpg border=off;
proc sgplot data=figERC1 DATTRMAP=Attrs2 noautolegend; TITLE;
SCATTER Y=Estimate X=q / YERRORLOWER=LowerCLMean YERRORUPPER=UpperCLMean
	ATTRID=Jobs2 markerattrs=(size=10 symbol=squarefilled color="BL") errorbarattrs=(thickness=2 color="BIGB" pattern=dash); 
refline 0 / lineattrs=( pattern=solid);
yaxis label='Incremental ERC' values=(-0.10 to 0.15 by 0.05) labelattrs=(size=13) valueattrs=(size=12);
xaxis label='Event Time Indicators (q-1 is base time period)' values=(-4 to 3 by 1) valuesdisplay=("q-4" "q-3" "q-2" "q-1" "q" "q+1" "q+2" "q+3") DISCRETEORDER=DATA labelattrs=(size=13) valueattrs=(size=12);
RUN;
ods listing close;


*******;
* Fig 2B - IPT;
*******;

data figIPT; length ETI $ 3; set parmIPT;
where Parameter in ("treatXpre4","treatXpre3","treatXpre2","treatXpost1","treatXpost2","treatXpost3","treatXpost4");
q=_n_ - 5; if q > -2 then q+1;
LowerCLMean = Estimate-(StdErr*2); UpperCLMean = Estimate+(StdErr*2);
if q < 0 then ETI= cats('q',put(q,2.)); else if q > 0 then ETI= cats('q+',put(q,2.)); else ETI='q';
drop DenDF tValue Probt;
run;

data figIPT1;length Parameter $ 16;set figIPT end=eof;output;
if eof then do; Parameter='treatXpre1'; q=-1; Estimate=0;StdErr=0;LowerCLMean=0;UpperCLMean=0;ETI='q-1';
output;end;run;
proc sort data=figIPT1; by q; run;

ods listing gpath="C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Analysis";
ods graphics on / reset=index imagename="Fig2IPT" imagefmt=jpg border=off;
proc sgplot data=figIPT1 DATTRMAP=Attrs2 noautolegend; TITLE;
SCATTER Y=Estimate X=q / YERRORLOWER=LowerCLMean YERRORUPPER=UpperCLMean
	ATTRID=Jobs2 markerattrs=(size=10 symbol=squarefilled color="BL") errorbarattrs=(thickness=2 color="BIGB" pattern=dash); 
refline 0 / lineattrs=( pattern=solid);
yaxis label='Incremental IPT' values=(-0.10 to 0.15 by 0.05) labelattrs=(size=13) valueattrs=(size=12);
xaxis label='Event Time Indicators (q-1 is base time period)' values=(-4 to 3 by 1) valuesdisplay=("q-4" "q-3" "q-2" "q-1" "q" "q+1" "q+2" "q+3") DISCRETEORDER=DATA labelattrs=(size=13) valueattrs=(size=12);
RUN;
ods listing close;


*******;
* Fig 2C - Abnormal Volume;
*******;

data figVOL; length ETI $ 3; set parmVOL;
where Parameter in ("treatXpre4","treatXpre3","treatXpre2","treatXpost1","treatXpost2","treatXpost3","treatXpost4");
q=_n_ - 5; if q > -2 then q+1;
LowerCLMean = Estimate-(StdErr*2); UpperCLMean = Estimate+(StdErr*2);
if q < 0 then ETI= cats('q',put(q,2.)); else if q > 0 then ETI= cats('q+',put(q,2.)); else ETI='q';
drop DenDF tValue Probt;
run;

data figVOL1;length Parameter $ 16;set figVOL end=eof;output;
if eof then do; Parameter='treatXpre1'; q=-1; Estimate=0;StdErr=0;LowerCLMean=0;UpperCLMean=0;ETI='q-1';
output;end;run;
proc sort data=figVOL1; by q; run;

ods listing gpath="C:\Users\nmg75\Dropbox (Personal)\Research\007 - Reporters as Analysts\Data\Analysis";
ods graphics on / reset=index imagename="Fig2VOL" imagefmt=jpg border=off;
proc sgplot data=figVOL1 DATTRMAP=Attrs2 noautolegend; TITLE;
SCATTER Y=Estimate X=q / YERRORLOWER=LowerCLMean YERRORUPPER=UpperCLMean
	ATTRID=Jobs2 markerattrs=(size=10 symbol=squarefilled color="BL") errorbarattrs=(thickness=2 color="BIGB" pattern=dash); 
refline 0 / lineattrs=( pattern=solid);
yaxis label='Incremental Abnormal Volume' values=(-0.10 to 0.15 by 0.05) labelattrs=(size=13) valueattrs=(size=12);
xaxis label='Event Time Indicators (q-1 is base time period)' values=(-4 to 3 by 1) valuesdisplay=("q-4" "q-3" "q-2" "q-1" "q" "q+1" "q+2" "q+3") DISCRETEORDER=DATA labelattrs=(size=13) valueattrs=(size=12);
RUN;
ods listing close;





